﻿Option Explicit On
Option Strict On

Imports System.Threading
Module GameServer

#Region "Variables"
    'SavePath
    'Private SavePath As String = Application.LocalUserAppDataPath()
    Private Delegate Sub SetTextDelegate(ByVal StatusText As String)
    'Save Files Location
    Private SavePath As String = System.Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) & "\SimTools\GameEngineSaveFiles"
    'Listening Ports
    Private Const GameManagerGui_Port_Listening As Integer = 65530
    Private Const GameEngineGui_Port_Listening As Integer = 65529
    Private Const AdminWindowGui_Port_Listening As Integer = 65528
    'UDP
    Private Const GameEngineGui_Port_Listening_UDP As Integer = 65529

    'Sending Ports
    Private Const GameEngineGui_Port_Sending As Integer = 65527
    Private Const AdminWindowGui_Port_Sending As Integer = 65526
    'UDP
    Private Const SendingUDPPort_GameManager As Integer = 65525
    Private Const SendingUDPPort_AdminWindow As Integer = 65524

    'Game Manager GUI
    Private MyTcpListener_GameManagerGui As New Mod_NetTools.NetClient
    Private Struct_ProfileData As Mod_NetTools.NetClient.ProfileData

    'Game Engine GUI
    Private MyTcpListener_GameEngineGui As New Mod_NetTools.NetClient
    Private MyTcpSender As New Mod_NetTools.NetClient
    Private MyUdpListener_OutputTesting As New Mod_NetTools.NetClient
    Private MyUdpSender_GameEngineGui As New Mod_NetTools.NetClient
    Private MyUdpSender_AdminWindow As New Mod_NetTools.NetClient
    Private Struct_AxisAssignments As Mod_NetTools.NetClient.AxisAssignments
    Private Struct_AxisLimiting As Mod_NetTools.NetClient.AxisLimiting
    Private Struct_OutputTesting As Mod_NetTools.NetClient.OutputTesting
    Private OutputTestingOn As Boolean = False
    Private Struct_InterfaceAssignments1 As Mod_NetTools.NetClient.InterfaceAssignments
    Private Struct_InterfaceAssignments2 As Mod_NetTools.NetClient.InterfaceAssignments
    Private Struct_InterfaceAssignments3 As Mod_NetTools.NetClient.InterfaceAssignments
    Private Struct_InterfaceAssignments4 As Mod_NetTools.NetClient.InterfaceAssignments
    Private Struct_InterfaceAssignments5 As Mod_NetTools.NetClient.InterfaceAssignments
    Private Struct_InterfaceAssignments6 As Mod_NetTools.NetClient.InterfaceAssignments

    'Game Admin GUI - uses the premade structs above
    Private Const AdminUpdateRateMS As Integer = 5
    Private MyTcpListener_AdminWindowGui As New Mod_NetTools.NetClient
    Private Struct_GameMaxMin As Mod_NetTools.NetClient.GameMaxMin
    Private Struct_GameAdminData As Mod_NetTools.NetClient.GameAdminData
    Private AdminOutputOn As Boolean = False
    Private AdminGUIConnected As Boolean = False

    'OutputTestingHelper
    Private LastLaunchedByGame As Boolean = True
#End Region

#Region "Form Open / Close"
    Sub Main()
        'see if we are already running
        Dim npProc() As Process
        npProc = Process.GetProcesses
        Dim x As Integer = 0
        For Each proc As Process In npProc
            If proc.ProcessName.StartsWith("Game_Engine") And proc.ProcessName.EndsWith("Game_Engine") Then
                x = x + 1
            ElseIf x = 2 Then
                MsgBox("Game Engine is already Running!" & vbCrLf & "", MsgBoxStyle.Exclamation)
                frm_Main.NotifyIcon1.Dispose()
                Application.Exit()
            End If
        Next

        AddHandler AppDomain.CurrentDomain.ProcessExit, AddressOf MyAppExit
        'Make sure Save Location Exists
        If System.IO.Directory.Exists(SavePath) = False Then
            System.IO.Directory.CreateDirectory(SavePath)
        End If
        'Start Servers
        CleanStructures()
        'load all saved structures
        LoadSavedStructs()
        Start_GameEngineGui_ServerTCP()
        Start_GameEngineGui_ServerUDP()
        Start_AdminWindow_ServerUDP()
        Start_GameManagerGui_Server()
        Start_AdminWindowGui_Server()
    End Sub

    Public Sub MyAppExit(sender As Object, e As System.EventArgs)
        'Stop Servers
        Mod_OutSim.Stop_OutSim_Listener()
        Mod_SimBin.Stop_SimBin_Listener()
        Mod_Calc.Axis_Calulations_OFF()
        Mod_Calc.Calculate_Percentges_OFF()
        Mod_Com.StopOutputToInterfaces()
        MyTcpListener_GameManagerGui.TcpClose()
        MyTcpListener_GameEngineGui.TcpClose()
        MyTcpListener_AdminWindowGui.TcpClose()
        MyUdpListener_OutputTesting.UDPClose()
        MyUdpSender_AdminWindow.UDPClose()
        MyUdpSender_GameEngineGui.UDPClose()
        MyTcpSender.TcpClose()
    End Sub
#End Region

#Region "GameManagerGui"
#Region "Listener"
    'Start TCP Server
    Private Sub Start_GameManagerGui_Server()
        AddHandler MyTcpListener_GameManagerGui.MessageWaiting, AddressOf GetMessageTcp_GM
        MyTcpListener_GameManagerGui.SetPort(GameManagerGui_Port_Listening)
        MyTcpListener_GameManagerGui.SetIp("Any")
        MyTcpListener_GameManagerGui.StartTCPListener()
    End Sub

    'This is where the message safely gets passed from one thread to another  
    Private Sub GetMessageTcp_GM()
        SetText_GM(MyTcpListener_GameManagerGui.GetMessage)
    End Sub

    'This is where the text actually gets set
    Private Sub SetText_GM(text As String)
        If text = "" Then
            Exit Sub
        End If
        If text.Contains("ProfileData") = True Then
            Struct_ProfileData = CType(MyTcpListener_GameManagerGui.DeSerializeString(text, "ProfileData"), NetClient.ProfileData)
            'Load Correct Max/Min Values Per Game
            Dim FinalPath As String = SavePath & "\" & Struct_ProfileData._GameName & ".MaxMin.cfg"
            If System.IO.File.Exists(FinalPath) Then
                Dim objReader As New System.IO.StreamReader(FinalPath)
                Dim SavedMaxMin As String = objReader.ReadToEnd()
                Struct_GameMaxMin = CType(MyTcpListener_AdminWindowGui.DeSerializeString(SavedMaxMin, "GameMaxMin"), NetClient.GameMaxMin)
                objReader.Close()
            End If
            If AdminGUIConnected = True Then
                SendTCP_AdminWindowGui(MyTcpSender.SerializeString("ProfileData", Struct_ProfileData))
            End If
        ElseIf text.Contains("GameStoped") = True Then
            'stop output
            'clear profile Data
            GameEndReset()
            If VirtualOutputRunning = True Then
                MyUdpSender_GameEngineGui.SendMessageUDP("TurnOffVirtualAxis")
                VirtualOutputOff()
            End If
        ElseIf text.Contains("GameStarted") = True Then
            'start output ####################################################
            Try
                If Mod_Calc.IsAnyOutputRequired = True Then
                    'shut all down to be safe 
                    'clear Data
                    Mod_Com.StopOutputToInterfaces()
                    Mod_OutSim.Stop_OutSim_Listener()
                    Mod_SimBin.Stop_SimBin_Listener()
                    Mod_Calc.Calculate_Percentges_OFF()
                    Mod_Calc.Axis_Calulations_OFF()
                    If LastLaunchedByGame = False Then
                        'Using the VR sender to turn off outputbutton
                        MyUdpSender_GameEngineGui.SendMessageUDP("TurnOffOutputTesting")
                    End If
                    'Structure GameAdminData - clear
                    Struct_GameAdminData._Axis1Output = 0
                    Struct_GameAdminData._Axis2Output = 0
                    Struct_GameAdminData._Axis3Output = 0
                    Struct_GameAdminData._Axis4Output = 0
                    Struct_GameAdminData._Axis5Output = 0
                    Struct_GameAdminData._Axis6Output = 0
                    Struct_GameAdminData._InputHeave = 0
                    Struct_GameAdminData._InputPitch = 0
                    Struct_GameAdminData._InputYaw = 0
                    Struct_GameAdminData._InputRoll = 0
                    Struct_GameAdminData._InputSurge = 0
                    Struct_GameAdminData._InputSway = 0
                    Struct_GameAdminData._OutputYaw = 0
                    Struct_GameAdminData._OutputSway = 0
                    Struct_GameAdminData._OutputSurge = 0
                    Struct_GameAdminData._OutputRoll = 0
                    Struct_GameAdminData._OutputPitch = 0
                    Struct_GameAdminData._OutputHeave = 0
                    'start or restart
                    LastLaunchedByGame = True

                    'Start the Right Listener 
                    If text.Contains("OutSim") = True Then 'What Listener to start
                        Mod_OutSim.Start_OutSim_Listener()
                    ElseIf text.Contains("SimBin") = True Then 'What Listener to start
                        Mod_SimBin.Start_SimBin_Listener()
                    ElseIf text.Contains("Generic") = True Then 'What Listener to start
                        'we are using SimBin as a generic input-er - it already stops too
                        Mod_SimBin.Start_SimBin_Listener()
                    End If
                    'Only Need Some #################             
                    Mod_Calc.StartupSelected()
                    '################################
                    Mod_Com.StartOutputToInterfaces()
                End If

            Catch ex As Exception
            End Try
            'start output ####################################################
        End If
    End Sub
#End Region

    Private Sub GameEndReset()
        Mod_Com.StopOutputToInterfaces()
        Mod_OutSim.Stop_OutSim_Listener()
        Mod_SimBin.Stop_SimBin_Listener()
        Mod_Calc.Calculate_Percentges_OFF()
        Mod_Calc.Axis_Calulations_OFF()
        Struct_ProfileData._GameName = "-"
        Struct_ProfileData._ProfileName = "-"
        Struct_ProfileData._MainLevel = 100
        Struct_ProfileData._RollEditor = 0
        Struct_ProfileData._RollClippingFilter = 0
        Struct_ProfileData._RollSmoothingFilter = 0
        Struct_ProfileData._RollWashoutFilter = 0
        Struct_ProfileData._RollWashoutFilter = 0
        Struct_ProfileData._PitchEditor = 0
        Struct_ProfileData._PitchClippingFilter = 0
        Struct_ProfileData._PitchSmoothingFilter = 0
        Struct_ProfileData._PitchWashoutFilter = 0
        Struct_ProfileData._PitchWashoutFilter = 0
        Struct_ProfileData._HeaveEditor = 0
        Struct_ProfileData._HeaveClippingFilter = 0
        Struct_ProfileData._HeaveSmoothingFilter = 0
        Struct_ProfileData._HeaveWashoutFilter = 0
        Struct_ProfileData._HeaveWashoutFilter = 0
        Struct_ProfileData._YawEditor = 0
        Struct_ProfileData._YawClippingFilter = 0
        Struct_ProfileData._YawSmoothingFilter = 0
        Struct_ProfileData._YawWashoutFilter = 0
        Struct_ProfileData._YawWashoutFilter = 0
        Struct_ProfileData._SwayEditor = 0
        Struct_ProfileData._SwayClippingFilter = 0
        Struct_ProfileData._SwaySmoothingFilter = 0
        Struct_ProfileData._SwayWashoutFilter = 0
        Struct_ProfileData._SwayWashoutFilter = 0
        Struct_ProfileData._SurgeEditor = 0
        Struct_ProfileData._SurgeClippingFilter = 0
        Struct_ProfileData._SurgeSmoothingFilter = 0
        Struct_ProfileData._SurgeWashoutFilter = 0
        Struct_ProfileData._SurgeWashoutFilter = 0
        'Structure GameAdminData
        Struct_GameAdminData._Axis1Output = 0
        Struct_GameAdminData._Axis2Output = 0
        Struct_GameAdminData._Axis3Output = 0
        Struct_GameAdminData._Axis4Output = 0
        Struct_GameAdminData._Axis5Output = 0
        Struct_GameAdminData._Axis6Output = 0
        Struct_GameAdminData._InputHeave = 0
        Struct_GameAdminData._InputPitch = 0
        Struct_GameAdminData._InputYaw = 0
        Struct_GameAdminData._InputRoll = 0
        Struct_GameAdminData._InputSurge = 0
        Struct_GameAdminData._InputSway = 0
        Struct_GameAdminData._OutputYaw = 0
        Struct_GameAdminData._OutputSway = 0
        Struct_GameAdminData._OutputSurge = 0
        Struct_GameAdminData._OutputRoll = 0
        Struct_GameAdminData._OutputPitch = 0
        Struct_GameAdminData._OutputHeave = 0
        If AdminGUIConnected = True Then
            SendTCP_AdminWindowGui(MyTcpSender.SerializeString("ProfileData", Struct_ProfileData))
        End If
    End Sub
#End Region

#Region "GameEngineGui"
#Region "Listener"
    'Incomming UDP Output Testing
    Private Sub SetText_GE_UDP(text As String)
        If text = "" Then
            Exit Sub
        ElseIf text.Contains("OutputTesting") = True Then
            If LastLaunchedByGame = True Then
                Exit Sub
            End If
            'if engen is not running
            Struct_OutputTesting = CType(MyUdpListener_OutputTesting.DeSerializeString(text, "OutputTesting"), NetClient.OutputTesting)
            If Struct_OutputTesting._OutputType = "DOF Output" Then
                If AxisCalcIsRunning() = False And OutputTestingOn = True Then
                    Mod_Calc.StartupSelected()
                End If
                Struct_GameAdminData._InputRoll = Struct_OutputTesting._Roll
                Struct_GameAdminData._InputPitch = Struct_OutputTesting._Pitch
                Struct_GameAdminData._InputHeave = Struct_OutputTesting._Heave
                Struct_GameAdminData._InputYaw = Struct_OutputTesting._Yaw
                Struct_GameAdminData._InputSway = Struct_OutputTesting._Sway
                Struct_GameAdminData._InputSurge = Struct_OutputTesting._Surge
                'Need Normal Output - Start Engine
            ElseIf Struct_OutputTesting._OutputType = "Axis Output" Then
                Mod_Calc.Axis_Calulations_OFF()
                Mod_Calc.Calculate_Percentges_OFF()
                Struct_GameAdminData._Axis1Output = Struct_OutputTesting._Axis1
                Struct_GameAdminData._Axis2Output = Struct_OutputTesting._Axis2
                Struct_GameAdminData._Axis3Output = Struct_OutputTesting._Axis3
                Struct_GameAdminData._Axis4Output = Struct_OutputTesting._Axis4
                Struct_GameAdminData._Axis5Output = Struct_OutputTesting._Axis5
                Struct_GameAdminData._Axis6Output = Struct_OutputTesting._Axis6
                'Need Partial Output - Start Engine
            ElseIf Struct_OutputTesting._OutputType = "Both" Then
                Struct_GameAdminData._InputRoll = Struct_OutputTesting._Roll
                Struct_GameAdminData._InputPitch = Struct_OutputTesting._Pitch
                Struct_GameAdminData._InputHeave = Struct_OutputTesting._Heave
                Struct_GameAdminData._InputYaw = Struct_OutputTesting._Yaw
                Struct_GameAdminData._InputSway = Struct_OutputTesting._Sway
                Struct_GameAdminData._InputSurge = Struct_OutputTesting._Surge
                Struct_GameAdminData._Axis1Output = Struct_OutputTesting._Axis1
                Struct_GameAdminData._Axis2Output = Struct_OutputTesting._Axis2
                Struct_GameAdminData._Axis3Output = Struct_OutputTesting._Axis3
                Struct_GameAdminData._Axis4Output = Struct_OutputTesting._Axis4
                Struct_GameAdminData._Axis5Output = Struct_OutputTesting._Axis5
                Struct_GameAdminData._Axis6Output = Struct_OutputTesting._Axis6
            End If
        End If
    End Sub

    Private Sub Start_GameEngineGui_ServerTCP()
        AddHandler MyTcpListener_GameEngineGui.MessageWaiting, AddressOf GetMessageTcp_GE
        MyTcpListener_GameEngineGui.SetPort(GameEngineGui_Port_Listening)
        MyTcpListener_GameEngineGui.SetIp("Any")
        MyTcpListener_GameEngineGui.StartTCPListener()
        StartUDPVirtualAxes()
    End Sub

    Private Sub Start_GameEngineGui_ServerUDP()
        AddHandler MyUdpListener_OutputTesting.MessageWaiting, AddressOf GetMessageUDP_GE
        MyUdpListener_OutputTesting.SetPort(GameEngineGui_Port_Listening_UDP)
        MyUdpListener_OutputTesting.SetIp("127.0.0.1")
        MyUdpListener_OutputTesting.StartUDPListener()
    End Sub

    Private Sub GetMessageTcp_GE()
        SetText_GE(MyTcpListener_GameEngineGui.GetMessage)
    End Sub

    Private Sub GetMessageUDP_GE()
        SetText_GE_UDP(MyUdpListener_OutputTesting.GetMessage)
    End Sub

    Private Sub StartUDPVirtualAxes()
        MyUdpSender_GameEngineGui.SetPort(SendingUDPPort_GameManager)
        MyUdpSender_GameEngineGui.SetIp("127.0.0.1")
        MyUdpSender_GameEngineGui.UDPOpen()
    End Sub

    'Virtual Output Here  #########################################################
    Private ThreadReceiveVO As Thread
    Private VirtualOutputRunning As Boolean = False

    Public Sub VirtualOutputOn()
        VirtualOutputRunning = True
        ThreadReceiveVO = New Thread(AddressOf VirtualOutput)
        ThreadReceiveVO.Start()
    End Sub

    Public Sub VirtualOutputOff()
        VirtualOutputRunning = False
    End Sub

    Private Sub VirtualOutput()
start:
        If VirtualOutputRunning = True Then
            MyUdpSender_GameEngineGui.SendMessageUDP(MyUdpSender_GameEngineGui.SerializeString("GameAdminData", Struct_GameAdminData))
        Else
            Exit Sub
        End If
        Thread.Sleep(20)
        GoTo start
    End Sub
    '##############################################################################

    'This is where the text actually gets set
    Private Sub SetText_GE(text As String)
        If text = "" Then
            Exit Sub
        ElseIf text = "SendSettings" Then
            SendTCP_GameEngineGui(MyTcpSender.SerializeString("AxisAssignments", Struct_AxisAssignments))
            SendTCP_GameEngineGui(MyTcpSender.SerializeString("AxisLimiting", Struct_AxisLimiting))
            SendInterfacesGE()
            Exit Sub
        ElseIf text.Contains("GetComList") = True Then
            Dim ComList As New ArrayList
            Dim OutComList As String = ""
            ComList = GetSerialPortNames()
            OutComList = "COMNameList"
            For i = 0 To ComList.Count - 1
                OutComList = OutComList & ","
                OutComList = OutComList & (CStr(ComList(i)))
            Next
            SendTCP_GameEngineGui(OutComList)
        ElseIf text.Contains("AxisAssignments") = True Then
            'Write file
            Struct_AxisAssignments = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "AxisAssignments"), NetClient.AxisAssignments)
            Dim FinalPath As String = SavePath & "\AxisAssignments.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("AxisAssignments", Struct_AxisAssignments))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendTCP_AdminWindowGui(MyTcpSender.SerializeString("AxisAssignments", Struct_AxisAssignments))
            End If
        ElseIf text.Contains("AxisLimiting") = True Then
            'Write file
            Struct_AxisLimiting = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "AxisLimiting"), NetClient.AxisLimiting)
            Dim FinalPath As String = SavePath & "\AxisLimiting.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("AxisLimiting", Struct_AxisLimiting))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendTCP_AdminWindowGui(MyTcpSender.SerializeString("AxisLimiting", Struct_AxisLimiting))
            End If
        ElseIf text.Contains("VirtualOutputOn") = True Then
            VirtualOutputOn()
        ElseIf text.Contains("VirtualOutputOff") = True Then
            VirtualOutputOff()
        ElseIf text.Contains("OutputTestingOn") = True Then
            If EngineIsRunning() = True Then
                'Using the UDP sender to turn off outputbutton
                MyUdpSender_GameEngineGui.SendMessageUDP("TurnOffOutputTesting")
                Exit Sub
            Else
                If Mod_Calc.IsAnyOutputRequired = True Then
                    OutputTestingOn = True
                    LastLaunchedByGame = False
                    Mod_Calc.StartupSelected()
                    Mod_Com.StartOutputToInterfaces()
                Else
                    MyUdpSender_GameEngineGui.SendMessageUDP("TurnOffOutputTesting")
                End If
            End If
            'Output Testing ###############################################################################################################
        ElseIf text.Contains("OutputTestingOff") = True Then
            OutputTestingOn = False
            If LastLaunchedByGame = False Then
                Mod_Com.StopOutputToInterfaces()
                GameEndReset()
            End If
            'Output Testing ###############################################################################################################
        ElseIf text.Contains("<_InterfaceNumber>1</_InterfaceNumber>") = True Then
            'Write file
            Struct_InterfaceAssignments1 = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "InterfaceAssignments"), NetClient.InterfaceAssignments)
            Dim FinalPath As String = SavePath & "\InterfaceAssignments1.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments1))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendInterfacesAW()
            End If
        ElseIf text.Contains("<_InterfaceNumber>2</_InterfaceNumber>") = True Then
            'Write file
            Struct_InterfaceAssignments2 = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "InterfaceAssignments"), NetClient.InterfaceAssignments)
            Dim FinalPath As String = SavePath & "\InterfaceAssignments2.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments2))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendInterfacesAW()
            End If
        ElseIf text.Contains("<_InterfaceNumber>3</_InterfaceNumber>") = True Then
            'Write file
            Struct_InterfaceAssignments3 = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "InterfaceAssignments"), NetClient.InterfaceAssignments)
            Dim FinalPath As String = SavePath & "\InterfaceAssignments3.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments3))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendInterfacesAW()
            End If
        ElseIf text.Contains("<_InterfaceNumber>4</_InterfaceNumber>") = True Then
            'Write file
            Struct_InterfaceAssignments4 = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "InterfaceAssignments"), NetClient.InterfaceAssignments)
            Dim FinalPath As String = SavePath & "\InterfaceAssignments4.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments4))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendInterfacesAW()
            End If
        ElseIf text.Contains("<_InterfaceNumber>5</_InterfaceNumber>") = True Then
            'Write file
            Struct_InterfaceAssignments5 = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "InterfaceAssignments"), NetClient.InterfaceAssignments)
            Dim FinalPath As String = SavePath & "\InterfaceAssignments5.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments5))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendInterfacesAW()
            End If
        ElseIf text.Contains("<_InterfaceNumber>6</_InterfaceNumber>") = True Then
            'Write file
            Struct_InterfaceAssignments6 = CType(MyTcpListener_GameEngineGui.DeSerializeString(text, "InterfaceAssignments"), NetClient.InterfaceAssignments)
            Dim FinalPath As String = SavePath & "\InterfaceAssignments6.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments6))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendInterfacesAW()
            End If
        End If
    End Sub
#End Region

#Region "SendTCP_GameEngineGui TCP"
    Private Sub SendTCP_GameEngineGui(text As String)
        MyTcpSender.SetPort(GameEngineGui_Port_Sending)
        MyTcpSender.SetIp("127.0.0.1")
        Dim DidOpen As Boolean = MyTcpSender.TcpOpen()
        If DidOpen = True Then
            MyTcpSender.SendMessageTCP(text)
            MyTcpSender.TcpClose()
        End If

    End Sub
#End Region
#End Region

#Region "AdminWindowGui"
#Region "Listener"
    Private Sub Start_AdminWindowGui_Server()
        AddHandler MyTcpListener_AdminWindowGui.MessageWaiting, AddressOf GetMessageTcp_AW
        MyTcpListener_AdminWindowGui.SetPort(AdminWindowGui_Port_Listening)
        MyTcpListener_AdminWindowGui.SetIp("Any")
        MyTcpListener_AdminWindowGui.StartTCPListener()
    End Sub

    Private Sub GetMessageTcp_AW()
        SetText_AW(MyTcpListener_AdminWindowGui.GetMessage)
    End Sub

    'This is where the text actually gets set
    Private Sub SetText_AW(text As String)
        If text = "" Then
            Exit Sub
        End If
        If text = "GetAdminData" Then
            'send structures
            SendTCP_AdminWindowGui(MyTcpSender.SerializeString("AxisLimiting", Struct_AxisLimiting))
            SendTCP_AdminWindowGui(MyTcpSender.SerializeString("AxisAssignments", Struct_AxisAssignments))
            SendInterfacesAW()
            AdminGUIConnected = True
        ElseIf text = "AdminSignOff" Then
            AdminGUIConnected = False
        ElseIf text = "AdminOutputOn" Then
            AdminOutputOn = True
            Struct_ProfileData._GameName = "AdminOutputOn"
            SendTCP_AdminWindowGui(MyTcpSender.SerializeString("ProfileData", Struct_ProfileData))
            'Start upd output
            OutputMainGameData()
        ElseIf text = "AdminOutputOff" Then
            'Stop upd output
            AdminOutputOn = False
        ElseIf text.Contains("GetGameMaxMin") = True Then
            If AdminGUIConnected = True Then
                Dim SplitText As String() = text.Split(New Char() {","c})
                Dim FinalPath As String = SavePath & "\" & SplitText(1) & ".MaxMin.cfg"
                If System.IO.File.Exists(FinalPath) Then
                    Dim objReader As New System.IO.StreamReader(FinalPath)
                    Dim SavedMaxMin As String = objReader.ReadToEnd()
                    Struct_GameMaxMin = CType(MyTcpListener_AdminWindowGui.DeSerializeString(SavedMaxMin, "GameMaxMin"), NetClient.GameMaxMin)
                    objReader.Close()
                    'Read File
                    If AdminGUIConnected = True Then
                        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("GameMaxMin", Struct_GameMaxMin))
                    End If
                Else
                    'Structure Max Min CLR
                    Struct_GameMaxMin._GameName = "-"
                    Struct_GameMaxMin._HeaveMax = 1
                    Struct_GameMaxMin._HeaveMin = -1
                    Struct_GameMaxMin._PitchMax = 1
                    Struct_GameMaxMin._PitchMin = -1
                    Struct_GameMaxMin._RollMax = 1
                    Struct_GameMaxMin._RollMin = -1
                    Struct_GameMaxMin._SurgeMax = 1
                    Struct_GameMaxMin._SurgeMin = -1
                    Struct_GameMaxMin._SwayMax = 1
                    Struct_GameMaxMin._SwayMin = -1
                    Struct_GameMaxMin._YawMax = 1
                    Struct_GameMaxMin._YawMin = -1
                    If AdminGUIConnected = True Then
                        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("GameMaxMin", Struct_GameMaxMin))
                    End If
                End If
            End If
        ElseIf text.Contains("GameMaxMin") = True Then
            'Save New Max/Min Limits
            Struct_GameMaxMin = CType(MyTcpListener_AdminWindowGui.DeSerializeString(text, "GameMaxMin"), NetClient.GameMaxMin)
            Dim FinalPath As String = SavePath & "\" & Struct_GameMaxMin._GameName & ".MaxMin.cfg"
            Dim objWriter As New System.IO.StreamWriter(FinalPath)
            objWriter.WriteLine(MyTcpSender.SerializeString("GameMaxMin", Struct_GameMaxMin))
            objWriter.Close()
            If AdminGUIConnected = True Then
                SendTCP_AdminWindowGui(MyTcpSender.SerializeString("GameMaxMin", Struct_GameMaxMin))
            End If
        End If
    End Sub

    Private Sub Start_AdminWindow_ServerUDP()
        MyUdpSender_AdminWindow.SetPort(SendingUDPPort_AdminWindow)
        MyUdpSender_AdminWindow.SetIp("127.0.0.1")
        MyUdpSender_AdminWindow.UDPOpen()
    End Sub

    '999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999
    Private Sub OutputMainGameData()
        Dim trd As Thread = New Thread(AddressOf sendData)
        trd.IsBackground = True
        trd.Start()
    End Sub

    Public Sub sendData()
myLoop:
        If AdminGUIConnected = True Then
            MyUdpSender_AdminWindow.SendMessageUDP(MyUdpSender_AdminWindow.SerializeString("GameAdminData", Struct_GameAdminData))
            Thread.Sleep(AdminUpdateRateMS)
        Else
            Exit Sub
        End If
        GoTo myLoop
    End Sub
    '999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999
#End Region

    Private Sub SendInterfacesAW()
        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments1))
        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments2))
        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments3))
        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments4))
        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments5))
        SendTCP_AdminWindowGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments6))
    End Sub

    Private Sub SendInterfacesGE()
        SendTCP_GameEngineGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments1))
        SendTCP_GameEngineGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments2))
        SendTCP_GameEngineGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments3))
        SendTCP_GameEngineGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments4))
        SendTCP_GameEngineGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments5))
        SendTCP_GameEngineGui(MyTcpSender.SerializeString("InterfaceAssignments", Struct_InterfaceAssignments6))
    End Sub

#Region "SendTCP_AdminWindowGui"
    Private Sub SendTCP_AdminWindowGui(text As String)
        MyTcpSender.SetPort(AdminWindowGui_Port_Sending)
        MyTcpSender.SetIp("127.0.0.1")
        Dim DidOpen As Boolean = MyTcpSender.TcpOpen()
        If DidOpen = True Then
            MyTcpSender.SendMessageTCP(text)
            MyTcpSender.TcpClose()
        End If
    End Sub
#End Region
#End Region

    Private Sub CleanStructures()
        'AxesLimiting
        Struct_AxisLimiting._Axis1Limiter = 100
        Struct_AxisLimiting._Axis2Limiter = 100
        Struct_AxisLimiting._Axis3Limiter = 100
        Struct_AxisLimiting._Axis4Limiter = 100
        Struct_AxisLimiting._Axis5Limiter = 100
        Struct_AxisLimiting._Axis6Limiter = 100

        'AxisAssignments
        Struct_AxisAssignments._Axis1DIR1 = False
        Struct_AxisAssignments._Axis1DIR2 = False
        Struct_AxisAssignments._Axis1DIR3 = False
        Struct_AxisAssignments._Axis2DIR1 = False
        Struct_AxisAssignments._Axis2DIR2 = False
        Struct_AxisAssignments._Axis2DIR3 = False
        Struct_AxisAssignments._Axis3DIR1 = False
        Struct_AxisAssignments._Axis3DIR2 = False
        Struct_AxisAssignments._Axis3DIR3 = False
        Struct_AxisAssignments._Axis4DIR1 = False
        Struct_AxisAssignments._Axis4DIR2 = False
        Struct_AxisAssignments._Axis4DIR3 = False
        Struct_AxisAssignments._Axis5DIR1 = False
        Struct_AxisAssignments._Axis5DIR2 = False
        Struct_AxisAssignments._Axis5DIR3 = False
        Struct_AxisAssignments._Axis6DIR1 = False
        Struct_AxisAssignments._Axis6DIR2 = False
        Struct_AxisAssignments._Axis6DIR3 = False
        Struct_AxisAssignments._Axis1DOF1 = "-"
        Struct_AxisAssignments._Axis1DOF2 = "-"
        Struct_AxisAssignments._Axis1DOF3 = "-"
        Struct_AxisAssignments._Axis2DOF1 = "-"
        Struct_AxisAssignments._Axis2DOF2 = "-"
        Struct_AxisAssignments._Axis2DOF3 = "-"
        Struct_AxisAssignments._Axis3DOF1 = "-"
        Struct_AxisAssignments._Axis3DOF2 = "-"
        Struct_AxisAssignments._Axis3DOF3 = "-"
        Struct_AxisAssignments._Axis4DOF1 = "-"
        Struct_AxisAssignments._Axis4DOF2 = "-"
        Struct_AxisAssignments._Axis4DOF3 = "-"
        Struct_AxisAssignments._Axis5DOF1 = "-"
        Struct_AxisAssignments._Axis5DOF2 = "-"
        Struct_AxisAssignments._Axis5DOF3 = "-"
        Struct_AxisAssignments._Axis6DOF1 = "-"
        Struct_AxisAssignments._Axis6DOF2 = "-"
        Struct_AxisAssignments._Axis6DOF3 = "-"
        Struct_AxisAssignments._Axis1PER1 = 0
        Struct_AxisAssignments._Axis1PER2 = 0
        Struct_AxisAssignments._Axis1PER3 = 0
        Struct_AxisAssignments._Axis2PER1 = 0
        Struct_AxisAssignments._Axis2PER2 = 0
        Struct_AxisAssignments._Axis2PER3 = 0
        Struct_AxisAssignments._Axis3PER1 = 0
        Struct_AxisAssignments._Axis3PER2 = 0
        Struct_AxisAssignments._Axis3PER3 = 0
        Struct_AxisAssignments._Axis4PER1 = 0
        Struct_AxisAssignments._Axis4PER2 = 0
        Struct_AxisAssignments._Axis4PER3 = 0
        Struct_AxisAssignments._Axis5PER1 = 0
        Struct_AxisAssignments._Axis5PER2 = 0
        Struct_AxisAssignments._Axis5PER3 = 0
        Struct_AxisAssignments._Axis6PER1 = 0
        Struct_AxisAssignments._Axis6PER2 = 0
        Struct_AxisAssignments._Axis6PER3 = 0

        'InterfaceAssignments
        Struct_InterfaceAssignments1._ComPort = "-"
        Struct_InterfaceAssignments1._BitsPerSec = 0
        Struct_InterfaceAssignments1._DataBits = 0
        Struct_InterfaceAssignments1._HWStartMS = 0
        Struct_InterfaceAssignments1._HWStopMS = 0
        Struct_InterfaceAssignments1._InterfaceNumber = 1
        Struct_InterfaceAssignments1._InterfaceOutput = "-"
        Struct_InterfaceAssignments1._OutputBits = 0
        Struct_InterfaceAssignments1._OutputType = "-"
        Struct_InterfaceAssignments1._PacketRateMS = 0
        Struct_InterfaceAssignments1._ParityBits = "-"
        Struct_InterfaceAssignments1._ShutdownOutput = "-"
        Struct_InterfaceAssignments1._StartupOutput = "-"
        Struct_InterfaceAssignments1._StopBits = "-"
        Struct_InterfaceAssignments2._ComPort = "-"
        Struct_InterfaceAssignments2._BitsPerSec = 0
        Struct_InterfaceAssignments2._DataBits = 0
        Struct_InterfaceAssignments2._HWStartMS = 0
        Struct_InterfaceAssignments2._HWStopMS = 0
        Struct_InterfaceAssignments2._InterfaceNumber = 2
        Struct_InterfaceAssignments2._InterfaceOutput = "-"
        Struct_InterfaceAssignments2._OutputBits = 0
        Struct_InterfaceAssignments2._OutputType = "-"
        Struct_InterfaceAssignments2._PacketRateMS = 0
        Struct_InterfaceAssignments2._ParityBits = "-"
        Struct_InterfaceAssignments2._ShutdownOutput = "-"
        Struct_InterfaceAssignments2._StartupOutput = "-"
        Struct_InterfaceAssignments2._StopBits = "-"
        Struct_InterfaceAssignments3._ComPort = "-"
        Struct_InterfaceAssignments3._BitsPerSec = 0
        Struct_InterfaceAssignments3._DataBits = 0
        Struct_InterfaceAssignments3._HWStartMS = 0
        Struct_InterfaceAssignments3._HWStopMS = 0
        Struct_InterfaceAssignments3._InterfaceNumber = 3
        Struct_InterfaceAssignments3._InterfaceOutput = "-"
        Struct_InterfaceAssignments3._OutputBits = 0
        Struct_InterfaceAssignments3._OutputType = "-"
        Struct_InterfaceAssignments3._PacketRateMS = 0
        Struct_InterfaceAssignments3._ParityBits = "-"
        Struct_InterfaceAssignments3._ShutdownOutput = "-"
        Struct_InterfaceAssignments3._StartupOutput = "-"
        Struct_InterfaceAssignments3._StopBits = "-"
        Struct_InterfaceAssignments4._ComPort = "-"
        Struct_InterfaceAssignments4._BitsPerSec = 0
        Struct_InterfaceAssignments4._DataBits = 0
        Struct_InterfaceAssignments4._HWStartMS = 0
        Struct_InterfaceAssignments4._HWStopMS = 0
        Struct_InterfaceAssignments4._InterfaceNumber = 4
        Struct_InterfaceAssignments4._InterfaceOutput = "-"
        Struct_InterfaceAssignments4._OutputBits = 0
        Struct_InterfaceAssignments4._OutputType = "-"
        Struct_InterfaceAssignments4._PacketRateMS = 0
        Struct_InterfaceAssignments4._ParityBits = "-"
        Struct_InterfaceAssignments4._ShutdownOutput = "-"
        Struct_InterfaceAssignments4._StartupOutput = "-"
        Struct_InterfaceAssignments4._StopBits = "-"
        Struct_InterfaceAssignments5._ComPort = "-"
        Struct_InterfaceAssignments5._BitsPerSec = 0
        Struct_InterfaceAssignments5._DataBits = 0
        Struct_InterfaceAssignments5._HWStartMS = 0
        Struct_InterfaceAssignments5._HWStopMS = 0
        Struct_InterfaceAssignments5._InterfaceNumber = 5
        Struct_InterfaceAssignments5._InterfaceOutput = "-"
        Struct_InterfaceAssignments5._OutputBits = 0
        Struct_InterfaceAssignments5._OutputType = "-"
        Struct_InterfaceAssignments5._PacketRateMS = 0
        Struct_InterfaceAssignments5._ParityBits = "-"
        Struct_InterfaceAssignments5._ShutdownOutput = "-"
        Struct_InterfaceAssignments5._StartupOutput = "-"
        Struct_InterfaceAssignments5._StopBits = "-"
        Struct_InterfaceAssignments6._ComPort = "-"
        Struct_InterfaceAssignments6._BitsPerSec = 0
        Struct_InterfaceAssignments6._DataBits = 0
        Struct_InterfaceAssignments6._HWStartMS = 0
        Struct_InterfaceAssignments6._HWStopMS = 0
        Struct_InterfaceAssignments6._InterfaceNumber = 6
        Struct_InterfaceAssignments6._InterfaceOutput = "-"
        Struct_InterfaceAssignments6._OutputBits = 0
        Struct_InterfaceAssignments6._OutputType = "-"
        Struct_InterfaceAssignments6._PacketRateMS = 0
        Struct_InterfaceAssignments6._ParityBits = "-"
        Struct_InterfaceAssignments6._ShutdownOutput = "-"
        Struct_InterfaceAssignments6._StartupOutput = "-"
        Struct_InterfaceAssignments6._StopBits = "-"

        'Structure Max Min
        Struct_GameMaxMin._GameName = "-"
        Struct_GameMaxMin._HeaveMax = 1
        Struct_GameMaxMin._HeaveMin = -1
        Struct_GameMaxMin._PitchMax = 1
        Struct_GameMaxMin._PitchMin = -1
        Struct_GameMaxMin._RollMax = 1
        Struct_GameMaxMin._RollMin = -1
        Struct_GameMaxMin._SurgeMax = 1
        Struct_GameMaxMin._SurgeMin = -1
        Struct_GameMaxMin._SwayMax = 1
        Struct_GameMaxMin._SwayMin = -1
        Struct_GameMaxMin._YawMax = 1
        Struct_GameMaxMin._YawMin = -1

        'Structure GameAdminData
        Struct_GameAdminData._Axis1Output = 0
        Struct_GameAdminData._Axis2Output = 0
        Struct_GameAdminData._Axis3Output = 0
        Struct_GameAdminData._Axis4Output = 0
        Struct_GameAdminData._Axis5Output = 0
        Struct_GameAdminData._Axis6Output = 0
        Struct_GameAdminData._InputHeave = 0
        Struct_GameAdminData._InputPitch = 0
        Struct_GameAdminData._InputYaw = 0
        Struct_GameAdminData._InputRoll = 0
        Struct_GameAdminData._InputSurge = 0
        Struct_GameAdminData._InputSway = 0
        Struct_GameAdminData._OutputYaw = 0
        Struct_GameAdminData._OutputSway = 0
        Struct_GameAdminData._OutputSurge = 0
        Struct_GameAdminData._OutputRoll = 0
        Struct_GameAdminData._OutputPitch = 0
        Struct_GameAdminData._OutputHeave = 0

        'Structure ProfileData   
        Struct_ProfileData._GameName = "-"
        Struct_ProfileData._ProfileName = "-"
        Struct_ProfileData._MainLevel = 100
        Struct_ProfileData._RollEditor = 0
        Struct_ProfileData._RollClippingFilter = 0
        Struct_ProfileData._RollSmoothingFilter = 0
        Struct_ProfileData._RollWashoutFilter = 0
        Struct_ProfileData._RollWashoutFilter = 0
        Struct_ProfileData._PitchEditor = 0
        Struct_ProfileData._PitchClippingFilter = 0
        Struct_ProfileData._PitchSmoothingFilter = 0
        Struct_ProfileData._PitchWashoutFilter = 0
        Struct_ProfileData._PitchWashoutFilter = 0
        Struct_ProfileData._HeaveEditor = 0
        Struct_ProfileData._HeaveClippingFilter = 0
        Struct_ProfileData._HeaveSmoothingFilter = 0
        Struct_ProfileData._HeaveWashoutFilter = 0
        Struct_ProfileData._HeaveWashoutFilter = 0
        Struct_ProfileData._YawEditor = 0
        Struct_ProfileData._YawClippingFilter = 0
        Struct_ProfileData._YawSmoothingFilter = 0
        Struct_ProfileData._YawWashoutFilter = 0
        Struct_ProfileData._YawWashoutFilter = 0
        Struct_ProfileData._SwayEditor = 0
        Struct_ProfileData._SwayClippingFilter = 0
        Struct_ProfileData._SwaySmoothingFilter = 0
        Struct_ProfileData._SwayWashoutFilter = 0
        Struct_ProfileData._SwayWashoutFilter = 0
        Struct_ProfileData._SurgeEditor = 0
        Struct_ProfileData._SurgeClippingFilter = 0
        Struct_ProfileData._SurgeSmoothingFilter = 0
        Struct_ProfileData._SurgeWashoutFilter = 0
        Struct_ProfileData._SurgeWashoutFilter = 0
    End Sub

    Private Sub LoadSavedStructs()
        Dim FinalPath As String = SavePath & "\AxisAssignments.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_AxisAssignments = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "AxisAssignments"), NetClient.AxisAssignments)
        End If
        FinalPath = SavePath & "\AxisLimiting.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_AxisLimiting = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "AxisLimiting"), NetClient.AxisLimiting)
        End If
        FinalPath = SavePath & "\InterfaceAssignments1.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_InterfaceAssignments1 = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "InterfaceAssignments"), NetClient.InterfaceAssignments)
        End If
        FinalPath = SavePath & "\InterfaceAssignments2.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_InterfaceAssignments2 = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "InterfaceAssignments"), NetClient.InterfaceAssignments)
        End If
        FinalPath = SavePath & "\InterfaceAssignments3.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_InterfaceAssignments3 = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "InterfaceAssignments"), NetClient.InterfaceAssignments)
        End If
        FinalPath = SavePath & "\InterfaceAssignments4.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_InterfaceAssignments4 = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "InterfaceAssignments"), NetClient.InterfaceAssignments)
        End If
        FinalPath = SavePath & "\InterfaceAssignments5.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_InterfaceAssignments5 = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "InterfaceAssignments"), NetClient.InterfaceAssignments)
        End If
        FinalPath = SavePath & "\InterfaceAssignments6.cfg"
        If System.IO.File.Exists(FinalPath) Then
            Dim objReader As New System.IO.StreamReader(FinalPath)
            Dim SerializedString As String = objReader.ReadToEnd()
            objReader.Close()
            Struct_InterfaceAssignments6 = CType(MyTcpListener_GameEngineGui.DeSerializeString(SerializedString, "InterfaceAssignments"), NetClient.InterfaceAssignments)
        End If
    End Sub

    '########################################################################################
#Region "Get and Set Subs and Functions"
#Region "Get/Set DOF Input Values - GameAdminData"
    'Set_InputValues
    Public Sub SetRoll(Value As Double)
        Struct_GameAdminData._InputRoll = Value
    End Sub

    Public Sub SetPitch(Value As Double)
        Struct_GameAdminData._InputPitch = Value
    End Sub

    Public Sub SetHeave(Value As Double)
        Struct_GameAdminData._InputHeave = Value
    End Sub

    Public Sub SetYaw(Value As Double)
        Struct_GameAdminData._InputYaw = Value
    End Sub

    Public Sub SetSway(Value As Double)
        Struct_GameAdminData._InputSway = Value
    End Sub

    Public Sub SetSurge(Value As Double)
        Struct_GameAdminData._InputSurge = Value
    End Sub

    'Get
    Public Function GetRoll() As Double
        Return Struct_GameAdminData._InputRoll
    End Function

    Public Function GetPitch() As Double
        Return Struct_GameAdminData._InputPitch
    End Function

    Public Function GetHeave() As Double
        Return Struct_GameAdminData._InputHeave
    End Function

    Public Function GetYaw() As Double
        Return Struct_GameAdminData._InputYaw
    End Function

    Public Function GetSway() As Double
        Return Struct_GameAdminData._InputSway
    End Function

    Public Function GetSurge() As Double
        Return Struct_GameAdminData._InputSurge
    End Function

    'Set_AxisMixingValues
    Public Sub Set_Axis1Output(Value As Double)
        Struct_GameAdminData._Axis1Output = Value
    End Sub

    Public Sub Set_Axis2Output(Value As Double)
        Struct_GameAdminData._Axis2Output = Value
    End Sub

    Public Sub Set_Axis3Output(Value As Double)
        Struct_GameAdminData._Axis3Output = Value
    End Sub

    Public Sub Set_Axis4Output(Value As Double)
        Struct_GameAdminData._Axis4Output = Value
    End Sub

    Public Sub Set_Axis5Output(Value As Double)
        Struct_GameAdminData._Axis5Output = Value
    End Sub

    Public Sub Set_Axis6Output(Value As Double)
        Struct_GameAdminData._Axis6Output = Value
    End Sub

    'Get_AxisMixingValues
    Public Function Get_Axis1Output() As Double
        Return Struct_GameAdminData._Axis1Output
    End Function

    Public Function Get_Axis2Output() As Double
        Return Struct_GameAdminData._Axis2Output
    End Function

    Public Function Get_Axis3Output() As Double
        Return Struct_GameAdminData._Axis3Output
    End Function

    Public Function Get_Axis4Output() As Double
        Return Struct_GameAdminData._Axis4Output
    End Function

    Public Function Get_Axis5Output() As Double
        Return Struct_GameAdminData._Axis5Output
    End Function

    Public Function Get_Axis6Output() As Double
        Return Struct_GameAdminData._Axis6Output
    End Function
#End Region

#Region "Set/Get DOF Ouput Percent Values - GameAdminData"
    'Set
    Public Sub SetRollPercent(Value As Double)
        Struct_GameAdminData._OutputRoll = Value
    End Sub

    Public Sub SetPitchPercent(Value As Double)
        Struct_GameAdminData._OutputPitch = Value
    End Sub

    Public Sub SetHeavePercent(Value As Double)
        Struct_GameAdminData._OutputHeave = Value
    End Sub

    Public Sub SetYawPercent(Value As Double)
        Struct_GameAdminData._OutputYaw = Value
    End Sub

    Public Sub SetSwayPercent(Value As Double)
        Struct_GameAdminData._OutputSway = Value
    End Sub

    Public Sub SetSurgePercent(Value As Double)
        Struct_GameAdminData._OutputSurge = Value
    End Sub

    'Get
    Public Function GetRollPercent() As Double
        Return Struct_GameAdminData._OutputRoll
    End Function

    Public Function GetPitchPercent() As Double
        Return Struct_GameAdminData._OutputPitch
    End Function

    Public Function GetHeavePercent() As Double
        Return Struct_GameAdminData._OutputHeave
    End Function

    Public Function GetYawPercent() As Double
        Return Struct_GameAdminData._OutputYaw
    End Function

    Public Function GetSwayPercent() As Double
        Return Struct_GameAdminData._OutputSway
    End Function

    Public Function GetSurgePercent() As Double
        Return Struct_GameAdminData._OutputSurge
    End Function
#End Region

#Region "Get Profile Editor Values - ProfileData"
    'get main
    Public Function GetEditorMain() As Double
        Return (100 - Struct_ProfileData._MainLevel)
    End Function
    'Get
    Public Function GetRollEditor() As Double
        Return Struct_ProfileData._RollEditor
    End Function

    Public Function GetPitchEditor() As Double
        Return Struct_ProfileData._PitchEditor
    End Function

    Public Function GetHeaveEditor() As Double
        Return Struct_ProfileData._HeaveEditor
    End Function

    Public Function GetYawEditor() As Double
        Return Struct_ProfileData._YawEditor
    End Function

    Public Function GetSwayEditor() As Double
        Return Struct_ProfileData._SwayEditor
    End Function

    Public Function GetSurgeEditor() As Double
        Return Struct_ProfileData._SurgeEditor
    End Function

    'get smoothing
    Public Function GetRollEditorSmothing() As Double
        Return Struct_ProfileData._RollSmoothingFilter
    End Function

    Public Function GetPitchEditorSmothing() As Double
        Return Struct_ProfileData._PitchSmoothingFilter
    End Function

    Public Function GetHeaveEditorSmothing() As Double
        Return Struct_ProfileData._HeaveSmoothingFilter
    End Function

    Public Function GetYawEditorSmothing() As Double
        Return Struct_ProfileData._YawSmoothingFilter
    End Function

    Public Function GetSwayEditorSmothing() As Double
        Return Struct_ProfileData._SwaySmoothingFilter
    End Function

    Public Function GetSurgeEditorSmothing() As Double
        Return Struct_ProfileData._SurgeSmoothingFilter
    End Function

    'get Washout
    Public Function GetRollEditorWashoutFilter() As Double
        Return Struct_ProfileData._RollWashoutFilter
    End Function

    Public Function GetPitchEditorWashoutFilter() As Double
        Return Struct_ProfileData._PitchWashoutFilter
    End Function

    Public Function GetHeaveEditorWashoutFilter() As Double
        Return Struct_ProfileData._HeaveWashoutFilter
    End Function

    Public Function GetYawEditorWashoutFilter() As Double
        Return Struct_ProfileData._YawWashoutFilter
    End Function

    Public Function GetSwayEditorWashoutFilter() As Double
        Return Struct_ProfileData._SwayWashoutFilter
    End Function

    Public Function GetSurgeEditorWashoutFilter() As Double
        Return Struct_ProfileData._SurgeWashoutFilter
    End Function
#End Region

#Region "Get DOF Max/Min Values - GameMaxMin"
    'Get
    Public Function GetRollMax() As Double
        Return Struct_GameMaxMin._RollMax
    End Function

    Public Function GetRollMin() As Double
        Return Struct_GameMaxMin._RollMin
    End Function

    Public Function GetPitchMax() As Double
        Return Struct_GameMaxMin._PitchMax
    End Function

    Public Function GetPitchMin() As Double
        Return Struct_GameMaxMin._PitchMin
    End Function

    Public Function GetHeaveMax() As Double
        Return Struct_GameMaxMin._HeaveMax
    End Function

    Public Function GetHeaveMin() As Double
        Return Struct_GameMaxMin._HeaveMin
    End Function

    Public Function GetYawMax() As Double
        Return Struct_GameMaxMin._YawMax
    End Function

    Public Function GetYawMin() As Double
        Return Struct_GameMaxMin._YawMin
    End Function

    Public Function GetSwayMax() As Double
        Return Struct_GameMaxMin._SwayMax
    End Function

    Public Function GetSwayMin() As Double
        Return Struct_GameMaxMin._SwayMin
    End Function

    Public Function GetSurgeMax() As Double
        Return Struct_GameMaxMin._SurgeMax
    End Function

    Public Function GetSurgeMin() As Double
        Return Struct_GameMaxMin._SurgeMin
    End Function
#End Region

#Region "Get AxisAssignments Values - AxisAssignments"
    'Get   
    Public Function GetAxisAssignments() As Object
        Return Struct_AxisAssignments
    End Function
#End Region

#Region "Get InterfaceAssignments Values - InterfaceAssignments"
    Public Function GetInterfaceAssignments1() As Object
        Return Struct_InterfaceAssignments1
    End Function

    Public Function GetInterfaceAssignments2() As Object
        Return Struct_InterfaceAssignments2
    End Function

    Public Function GetInterfaceAssignments3() As Object
        Return Struct_InterfaceAssignments3
    End Function

    Public Function GetInterfaceAssignments4() As Object
        Return Struct_InterfaceAssignments4
    End Function

    Public Function GetInterfaceAssignments5() As Object
        Return Struct_InterfaceAssignments5
    End Function

    Public Function GetInterfaceAssignments6() As Object
        Return Struct_InterfaceAssignments6
    End Function
#End Region

#Region "Get AxisLimiting Values - AxisLimiting"
    Public Function GetAxisLimiting_Axis1() As Double
        Return Struct_AxisLimiting._Axis1Limiter
    End Function

    Public Function GetAxisLimiting_Axis2() As Double
        Return Struct_AxisLimiting._Axis2Limiter
    End Function

    Public Function GetAxisLimiting_Axis3() As Double
        Return Struct_AxisLimiting._Axis3Limiter
    End Function

    Public Function GetAxisLimiting_Axis4() As Double
        Return Struct_AxisLimiting._Axis4Limiter
    End Function

    Public Function GetAxisLimiting_Axis5() As Double
        Return Struct_AxisLimiting._Axis5Limiter
    End Function

    Public Function GetAxisLimiting_Axis6() As Double
        Return Struct_AxisLimiting._Axis6Limiter
    End Function
#End Region

#End Region
    '########################################################################################
End Module
