﻿Option Explicit On
Option Strict On

Imports System.IO.Ports
Imports System.Threading
Imports System.Math
Imports System.Text

Module Mod_Com
#Region "ComWorker"
    Public Class ComWorker
        Public Event MsgRecieved(msg As String)
#Region "Manager Variables"
        'property
        Private _baudRate As String = "115200"
        Private _parity As String = "None"
        Private _stopBits As String = "1"
        Private _dataBits As String = "8"
        Private _portName As String = "4123"

        'global
        Private comPort As New SerialPort()
        Private write As Boolean = True

#End Region

#Region "Manager Properties"
        'BaudRate
        Public Property BaudRate() As String
            Get
                Return _baudRate
            End Get
            Set(ByVal value As String)
                _baudRate = value
            End Set
        End Property

        'Parity
        Public Property Parity() As String
            Get
                Return _parity
            End Get
            Set(ByVal value As String)
                _parity = value
            End Set
        End Property

        'StopBits
        Public Property StopBits() As String
            Get
                Return _stopBits
            End Get
            Set(ByVal value As String)
                _stopBits = value
            End Set
        End Property

        'DataBits
        Public Property DataBits() As String
            Get
                Return _dataBits
            End Get
            Set(ByVal value As String)
                _dataBits = value
            End Set
        End Property

        'PortName
        Public Property PortName() As String
            Get
                Return _portName
            End Get
            Set(ByVal value As String)
                _portName = value
            End Set
        End Property
#End Region

#Region "OpenPort"
        Public Function OpenPort() As Boolean
            'load CHR codes for faster output
            If AsciiCodes.Count = 0 Then
                AsciiCodes.Clear()
                For x = 0 To 255
                    AsciiCodes.Add(Chr(x))
                Next
            End If
            Try
                'check if the port is open
                If comPort.IsOpen = True Then
                    comPort.Close()
                End If
                'SerialPort Object Properties
                'ComPort
                comPort.BaudRate = Integer.Parse(_baudRate)
                'DataBits
                comPort.DataBits = Integer.Parse(_dataBits)
                'StopBits
                comPort.StopBits = DirectCast([Enum].Parse(GetType(StopBits), _stopBits), StopBits)
                'Parity
                comPort.Parity = DirectCast([Enum].Parse(GetType(Parity), _parity), Parity)
                'PortName
                comPort.PortName = _portName
                'open port             
                comPort.Open()
                'display message
                'RaiseEvent MsgRecieved("Port opened at " + DateTime.Now + "" + Environment.NewLine + "")
                'if opened return true
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function
#End Region

#Region "ClosePort "
        Public Sub ClosePort()
            If comPort.IsOpen Then
                'RaiseEvent MsgRecieved("Port closed at " + DateTime.Now + "" + Environment.NewLine + "")
                comPort.Close()
            End If
        End Sub
#End Region

#Region "WriteData"
        Public Function WriteData(ByVal msg As String) As Boolean
            comPort.Encoding = Encoding.Default
            'port is open?
            If Not (comPort.IsOpen = True) Then
                comPort.Open()
            End If
            Try
                'send message
                comPort.Write(msg)
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function
#End Region

#Region "ReceivedData"
        Public Sub comPort_DataReceived(ByVal sender As Object, ByVal e As SerialDataReceivedEventArgs)
            Try
                Dim msg As String = comPort.ReadExisting()
                'return the data
                RaiseEvent MsgRecieved(msg)
            Catch ex As Exception
            End Try
        End Sub
#End Region
    End Class
#End Region

    Private AsciiCodes As New ArrayList
    'Functions for interface
    Private Function GetChr(ByVal CharCode As Double) As String
        Dim ChrStringOut As String = ""
        If CharCode > 16777215 Then
            Dim Multiplierx1 As Double = CharCode / 16777216
            If Multiplierx1 > 255 Then
                Multiplierx1 = 255
            End If
            Dim TempString() As String = Split(CStr(Multiplierx1), ".")
            Try
                CharCode = CharCode - (CDbl(TempString(0)) * 16777216)
            Catch ex As Exception
                CharCode = 0
            End Try
            ChrStringOut = CStr(AsciiCodes(CInt(TempString(0))))
        End If
        If CharCode > 65535 Then
            Dim Multiplierx1 As Double = CharCode / 65536
            If Multiplierx1 > 255 Then
                Multiplierx1 = 255
            End If
            Dim TempString() As String = Split(CStr(Multiplierx1), ".")
            Try
                CharCode = CharCode - (CDbl(TempString(0)) * 65536)
            Catch ex As Exception
                CharCode = 0
            End Try
            ChrStringOut = ChrStringOut & CStr(AsciiCodes(CInt(TempString(0))))
        End If
        If CharCode > 255 Then
            Dim Multiplierx1 As Double = CharCode / 256.0
            If Multiplierx1 > 255 Then
                Multiplierx1 = 255
            End If
            Dim TempString() As String = Split(CStr(Multiplierx1), ".")
            Try
                CharCode = CharCode - (CDbl(TempString(0)) * 256)
            Catch ex As Exception
                CharCode = 0
            End Try
            ChrStringOut = ChrStringOut & CStr(AsciiCodes(CInt(TempString(0))))
        End If
        ChrStringOut = ChrStringOut & CStr(AsciiCodes(CInt(CharCode)))
        Return ChrStringOut
    End Function

    Private Function GetHex(ByVal HexCode As Double) As String
        Return CStr(Conversion.Hex(HexCode))
    End Function

    Private Function GetOutPut(InputPercent As Double, BitsNeeded As String, Type As String) As String
        Dim OutPut As Double = 0
        Dim FinalOutput As String = ""

        'Bits
        Select Case BitsNeeded
            Case "7"
                'Positives are one higher - then we add the bottom amount
                '64 + 63 = 127 7bit
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 64) + 63
                ElseIf InputPercent < 0 Then
                    OutPut = 63 - (InputPercent * -63)
                Else
                    'middle
                    OutPut = 63
                End If
            Case "8"
                'Positives are one higher - then we add the bottom amount
                '128 + 127 = 255 8bit
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 128) + 127
                ElseIf InputPercent < 0 Then
                    OutPut = 127 - (InputPercent * -127)
                Else
                    'middle
                    OutPut = 127
                End If
            Case "9"
                '511
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 256) + 255
                ElseIf InputPercent < 0 Then
                    OutPut = 255 - (InputPercent * -255)
                Else
                    OutPut = 255
                End If
            Case "10"
                '1023
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 512) + 511
                ElseIf InputPercent < 0 Then
                    OutPut = 511 - (InputPercent * -511)
                Else
                    OutPut = 511
                End If
            Case "11"
                '2047
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 1024) + 1023
                ElseIf InputPercent < 0 Then
                    OutPut = 1023 - (InputPercent * -1023)
                Else
                    OutPut = 1023
                End If
            Case "12"
                '4095
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 2048) + 2047
                ElseIf InputPercent < 0 Then
                    OutPut = 2047 - (InputPercent * -2047)
                Else
                    OutPut = 2047
                End If
            Case "13"
                '8191
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 4096) + 4095
                ElseIf InputPercent < 0 Then
                    OutPut = 4095 - (InputPercent * -4095)
                Else
                    OutPut = 4095
                End If
            Case "14"
                '16283
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 8142) + 8141
                ElseIf InputPercent < 0 Then
                    OutPut = 8141 - (InputPercent * -8141)
                Else
                    OutPut = 8141
                End If
            Case "15"
                '32767
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 16384) + 16383
                ElseIf InputPercent < 0 Then
                    OutPut = 16383 - (InputPercent * -16383)
                Else
                    OutPut = 16383
                End If
            Case "16"
                '65535
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 32768) + 32767
                ElseIf InputPercent < 0 Then
                    OutPut = 32767 - (InputPercent * -32767)
                Else
                    OutPut = 32767
                End If
            Case "17"
                '131071
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 65536) + 65535
                ElseIf InputPercent < 0 Then
                    OutPut = 65535 - (InputPercent * -65535)
                Else
                    OutPut = 65535
                End If
            Case "18"
                '262143
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 131072) + 131071
                ElseIf InputPercent < 0 Then
                    OutPut = 131071 - (InputPercent * -131071)
                Else
                    OutPut = 131071
                End If
            Case "19"
                '524287
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 262144) + 262143
                ElseIf InputPercent < 0 Then
                    OutPut = 262143 - (InputPercent * -262143)
                Else
                    OutPut = 262143
                End If
            Case "20"
                '1048575
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 524288) + 524287
                ElseIf InputPercent < 0 Then
                    OutPut = 524287 - (InputPercent * -524287)
                Else
                    OutPut = 524287
                End If
            Case "21"
                '2097151
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 1048576) + 1048575
                ElseIf InputPercent < 0 Then
                    OutPut = 1048575 - (InputPercent * -1048575)
                Else
                    OutPut = 1048575
                End If
            Case "22"
                '4194303
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 2097152) + 2097151
                ElseIf InputPercent < 0 Then
                    OutPut = 2097151 - (InputPercent * -2097151)
                Else
                    OutPut = 2097151
                End If
            Case "23"
                '8388607
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 4194304) + 4194303
                ElseIf InputPercent < 0 Then
                    OutPut = 4194303 - (InputPercent * -4194303)
                Else
                    OutPut = 4194303
                End If
            Case "24"
                '16777215
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 8388608) + 8388607
                ElseIf InputPercent < 0 Then
                    OutPut = 8388607 - (InputPercent * -8388607)
                Else
                    OutPut = 8388607
                End If
            Case "25"
                '33554431
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 16777216) + 16777215
                ElseIf InputPercent < 0 Then
                    OutPut = 16777215 - (InputPercent * -16777215)
                Else
                    OutPut = 16777215
                End If
            Case "26"
                '67108863
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 33554432) + 33554431
                ElseIf InputPercent < 0 Then
                    OutPut = 33554431 - (InputPercent * -33554431)
                Else
                    OutPut = 33554431
                End If
            Case "27"
                '134217727
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 67108864) + 67108863
                ElseIf InputPercent < 0 Then
                    OutPut = 67108863 - (InputPercent * -67108863)
                Else
                    OutPut = 67108863
                End If
            Case "28"
                '268435455
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 134217728) + 134217727
                ElseIf InputPercent < 0 Then
                    OutPut = 134217727 - (InputPercent * -134217727)
                Else
                    OutPut = 134217727
                End If
            Case "29"
                '536870911
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 268435456) + 268435455
                ElseIf InputPercent < 0 Then
                    OutPut = 268435455 - (InputPercent * -268435455)
                Else
                    OutPut = 268435455
                End If
            Case "30"
                '1073741823
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 536870912) + 536870911
                ElseIf InputPercent < 0 Then
                    OutPut = 536870911 - (InputPercent * -536870911)
                Else
                    OutPut = 536870911
                End If
            Case "31"
                '2147483647
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 1073741824) + 1073741823
                ElseIf InputPercent < 0 Then
                    OutPut = 1073741823 - (InputPercent * -1073741823)
                Else
                    OutPut = 1073741823
                End If
            Case "32"
                '4294967295
                If InputPercent > 0 Then
                    OutPut = (InputPercent * 2147483648) + 2147483647
                ElseIf InputPercent < 0 Then
                    OutPut = 2147483647 - (InputPercent * -2147483647)
                Else
                    OutPut = 2147483647
                End If
        End Select

        'Convert to an integer
        OutPut = Math.Round(OutPut, 0)
        'If OutPut = 513 Then
        '    MsgBox("works")
        'End If
        'Type
        Select Case Type
            Case "Binary"
                FinalOutput = GetChr(OutPut)
                'output same number of char's always
                If CInt(BitsNeeded) > 8 Then
                    If FinalOutput.Length = 1 Then
                        FinalOutput = CStr(AsciiCodes(0)) & FinalOutput
                    End If
                End If
                If CInt(BitsNeeded) > 16 Then
                    If FinalOutput.Length = 2 Then
                        FinalOutput = CStr(AsciiCodes(0)) & FinalOutput
                    End If
                End If
                If CInt(BitsNeeded) > 24 Then
                    If FinalOutput.Length = 3 Then
                        FinalOutput = CStr(AsciiCodes(0)) & FinalOutput
                    End If
                End If
            Case "Hex"
                FinalOutput = GetHex(OutPut)
            Case "Decimal"
                FinalOutput = CStr(OutPut)
        End Select
        Return FinalOutput
    End Function

    'Function For Game Engene Gui
    Public ComList As New ArrayList
    Public Function GetSerialPortNames() As ArrayList
        ' Get a list of serial port names. 
        Dim ports As String() = SerialPort.GetPortNames()
        ' Display each port name to the console. 
        Dim port As String
        ComList.Clear()
        For Each port In ports
            ComList.Add(port)
        Next port
        Return ComList
    End Function


#Region "Startup Interfaces Here"
    Private InterfacesRunning As Boolean = False
    Public Sub StartOutputToInterfaces()
        InterfacesRunning = True

        'Interface 1
        Dim trd1 As Thread = New Thread(AddressOf Interface1Startup)
        trd1.IsBackground = True
        trd1.Start()

        'Interface 2
        Dim trd2 As Thread = New Thread(AddressOf Interface2Startup)
        trd2.IsBackground = True
        trd2.Start()

        'Interface 3
        Dim trd3 As Thread = New Thread(AddressOf Interface3Startup)
        trd3.IsBackground = True
        trd3.Start()

        'Interface 4
        Dim trd4 As Thread = New Thread(AddressOf Interface4Startup)
        trd4.IsBackground = True
        trd4.Start()

        'Interface 5
        Dim trd5 As Thread = New Thread(AddressOf Interface5Startup)
        trd5.IsBackground = True
        trd5.Start()

        'Interface 6
        Dim trd6 As Thread = New Thread(AddressOf Interface6Startup)
        trd6.IsBackground = True
        trd6.Start()
    End Sub

    'Stop all Interfaces Here
    Public Sub StopOutputToInterfaces()
        InterfacesRunning = False
    End Sub

    Public Function AreInterfacesRunning() As Boolean
        Return InterfacesRunning
    End Function

    Private Interface1_InterfaceAssignments As Mod_NetTools.NetClient.InterfaceAssignments
    Private Interface1_ComWorker As New Mod_Com.ComWorker
    Private Sub Interface1Startup()
        'Get Assignments
        Interface1_InterfaceAssignments = CType(GameServer.GetInterfaceAssignments1, NetClient.InterfaceAssignments)
        'See if axis is used
        If Interface1_InterfaceAssignments._ComPort = "-" Then
            'not used
            Exit Sub
        End If

        'Fix Output Strings - <ascii codes from number> - get ascii numbers
        Interface1_InterfaceAssignments._StartupOutput = ReplaceWithAsciiCode(Interface1_InterfaceAssignments._StartupOutput)
        Interface1_InterfaceAssignments._InterfaceOutput = ReplaceWithAsciiCode(Interface1_InterfaceAssignments._InterfaceOutput)
        Interface1_InterfaceAssignments._ShutdownOutput = ReplaceWithAsciiCode(Interface1_InterfaceAssignments._ShutdownOutput)

        'COM Port Assignments - set assignments to the com port
        Interface1_ComWorker.PortName = Interface1_InterfaceAssignments._ComPort
        Interface1_ComWorker.BaudRate = CStr(Interface1_InterfaceAssignments._BitsPerSec)
        Interface1_ComWorker.DataBits = CStr(Interface1_InterfaceAssignments._DataBits)
        Interface1_ComWorker.Parity = Interface1_InterfaceAssignments._ParityBits
        Interface1_ComWorker.StopBits = Interface1_InterfaceAssignments._StopBits
        'See if it opens
        If Interface1_ComWorker.OpenPort = True Then
            'We need to manipulate the final output string
            Dim Output As String
            'Startup
            Interface1_ComWorker.WriteData(Interface1_InterfaceAssignments._StartupOutput)
            Thread.Sleep(Interface1_InterfaceAssignments._HWStartMS)
            While InterfacesRunning = True
                Output = Interface1_InterfaceAssignments._InterfaceOutput
                If Output.Contains("<Axis1>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis1Output, Interface1_InterfaceAssignments._OutputBits.ToString, Interface1_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis1>", FinalOut)
                End If
                If Output.Contains("<Axis2>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis2Output, Interface1_InterfaceAssignments._OutputBits.ToString, Interface1_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis2>", FinalOut)
                End If
                If Output.Contains("<Axis3>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis3Output, Interface1_InterfaceAssignments._OutputBits.ToString, Interface1_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis3>", FinalOut)
                End If
                If Output.Contains("<Axis4>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis4Output, Interface1_InterfaceAssignments._OutputBits.ToString, Interface1_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis4>", FinalOut)
                End If
                If Output.Contains("<Axis5>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis5Output, Interface1_InterfaceAssignments._OutputBits.ToString, Interface1_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis5>", FinalOut)
                End If
                If Output.Contains("<Axis6>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis6Output, Interface1_InterfaceAssignments._OutputBits.ToString, Interface1_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis6>", FinalOut)
                End If
                'Send Output
                'Output & Chr(13)
                Interface1_ComWorker.WriteData(Output)
                'Sleep till next packet is needed
                Thread.Sleep(Interface1_InterfaceAssignments._PacketRateMS)
            End While
            'Shutdown
            Thread.Sleep(Interface1_InterfaceAssignments._HWStopMS)
            Interface1_ComWorker.WriteData(Interface1_InterfaceAssignments._ShutdownOutput)
            Interface1_ComWorker.ClosePort()
        Else
            'did not open
            Console.Beep(1000, 100)
            Console.Beep()
            'MsgBox("Interface1 could Not Start" & vbCrLf & "Please review your settings")
        End If
    End Sub

    Private Interface2_InterfaceAssignments As Mod_NetTools.NetClient.InterfaceAssignments
    Private Interface2_ComWorker As New Mod_Com.ComWorker
    Private Sub Interface2Startup()
        'Get Assignments
        Interface2_InterfaceAssignments = CType(GameServer.GetInterfaceAssignments2, NetClient.InterfaceAssignments)
        'See if axis is used
        If Interface2_InterfaceAssignments._ComPort = "-" Then
            'not used
            Exit Sub
        End If

        'Fix Output Strings - <ascii codes from number> - get ascii numbers
        Interface2_InterfaceAssignments._StartupOutput = ReplaceWithAsciiCode(Interface2_InterfaceAssignments._StartupOutput)
        Interface2_InterfaceAssignments._InterfaceOutput = ReplaceWithAsciiCode(Interface2_InterfaceAssignments._InterfaceOutput)
        Interface2_InterfaceAssignments._ShutdownOutput = ReplaceWithAsciiCode(Interface2_InterfaceAssignments._ShutdownOutput)

        'COM Port Assignments - set assignments to the com port
        Interface2_ComWorker.PortName = Interface2_InterfaceAssignments._ComPort
        Interface2_ComWorker.BaudRate = CStr(Interface2_InterfaceAssignments._BitsPerSec)
        Interface2_ComWorker.DataBits = CStr(Interface2_InterfaceAssignments._DataBits)
        Interface2_ComWorker.Parity = Interface2_InterfaceAssignments._ParityBits
        Interface2_ComWorker.StopBits = Interface2_InterfaceAssignments._StopBits
        'See if it opens
        If Interface2_ComWorker.OpenPort = True Then
            'We need to manipulate the final output string
            Dim Output As String
            'Startup
            Interface2_ComWorker.WriteData(Interface2_InterfaceAssignments._StartupOutput)
            Thread.Sleep(Interface2_InterfaceAssignments._HWStartMS)
            While InterfacesRunning = True
                Output = Interface2_InterfaceAssignments._InterfaceOutput
                If Output.Contains("<Axis1>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis1Output, Interface2_InterfaceAssignments._OutputBits.ToString, Interface2_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis1>", FinalOut)
                End If
                If Output.Contains("<Axis2>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis2Output, Interface2_InterfaceAssignments._OutputBits.ToString, Interface2_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis2>", FinalOut)
                End If
                If Output.Contains("<Axis3>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis3Output, Interface2_InterfaceAssignments._OutputBits.ToString, Interface2_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis3>", FinalOut)
                End If
                If Output.Contains("<Axis4>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis4Output, Interface2_InterfaceAssignments._OutputBits.ToString, Interface2_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis4>", FinalOut)
                End If
                If Output.Contains("<Axis5>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis5Output, Interface2_InterfaceAssignments._OutputBits.ToString, Interface2_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis5>", FinalOut)
                End If
                If Output.Contains("<Axis6>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis6Output, Interface2_InterfaceAssignments._OutputBits.ToString, Interface2_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis6>", FinalOut)
                End If
                'Send Output
                Interface2_ComWorker.WriteData(Output)
                'Sleep till next packet is needed
                Thread.Sleep(Interface2_InterfaceAssignments._PacketRateMS)
            End While
            'Shutdown
            Thread.Sleep(Interface2_InterfaceAssignments._HWStopMS)
            Interface2_ComWorker.WriteData(Interface2_InterfaceAssignments._ShutdownOutput)
            Interface2_ComWorker.ClosePort()
        Else
            'did not open
            Console.Beep(1000, 100)
            Console.Beep()
            'MsgBox("Interface1 could Not Start" & vbCrLf & "Please review your settings")
        End If
    End Sub

    Private Interface3_InterfaceAssignments As Mod_NetTools.NetClient.InterfaceAssignments
    Private Interface3_ComWorker As New Mod_Com.ComWorker
    Private Sub Interface3Startup()
        'Get Assignments
        Interface3_InterfaceAssignments = CType(GameServer.GetInterfaceAssignments3, NetClient.InterfaceAssignments)
        'See if axis is used
        If Interface3_InterfaceAssignments._ComPort = "-" Then
            'not used
            Exit Sub
        End If

        'Fix Output Strings - <ascii codes from number> - get ascii numbers
        Interface3_InterfaceAssignments._StartupOutput = ReplaceWithAsciiCode(Interface3_InterfaceAssignments._StartupOutput)
        Interface3_InterfaceAssignments._InterfaceOutput = ReplaceWithAsciiCode(Interface3_InterfaceAssignments._InterfaceOutput)
        Interface3_InterfaceAssignments._ShutdownOutput = ReplaceWithAsciiCode(Interface3_InterfaceAssignments._ShutdownOutput)

        'COM Port Assignments - set assignments to the com port
        Interface3_ComWorker.PortName = Interface3_InterfaceAssignments._ComPort
        Interface3_ComWorker.BaudRate = CStr(Interface3_InterfaceAssignments._BitsPerSec)
        Interface3_ComWorker.DataBits = CStr(Interface3_InterfaceAssignments._DataBits)
        Interface3_ComWorker.Parity = Interface3_InterfaceAssignments._ParityBits
        Interface3_ComWorker.StopBits = Interface3_InterfaceAssignments._StopBits
        'See if it opens
        If Interface3_ComWorker.OpenPort = True Then
            'We need to manipulate the final output string
            Dim Output As String
            'Startup
            Interface3_ComWorker.WriteData(Interface3_InterfaceAssignments._StartupOutput)
            Thread.Sleep(Interface3_InterfaceAssignments._HWStartMS)
            While InterfacesRunning = True
                Output = Interface3_InterfaceAssignments._InterfaceOutput
                If Output.Contains("<Axis1>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis1Output, Interface3_InterfaceAssignments._OutputBits.ToString, Interface3_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis1>", FinalOut)
                End If
                If Output.Contains("<Axis2>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis2Output, Interface3_InterfaceAssignments._OutputBits.ToString, Interface3_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis2>", FinalOut)
                End If
                If Output.Contains("<Axis3>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis3Output, Interface3_InterfaceAssignments._OutputBits.ToString, Interface3_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis3>", FinalOut)
                End If
                If Output.Contains("<Axis4>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis4Output, Interface3_InterfaceAssignments._OutputBits.ToString, Interface3_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis4>", FinalOut)
                End If
                If Output.Contains("<Axis5>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis5Output, Interface3_InterfaceAssignments._OutputBits.ToString, Interface3_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis5>", FinalOut)
                End If
                If Output.Contains("<Axis6>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis6Output, Interface3_InterfaceAssignments._OutputBits.ToString, Interface3_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis6>", FinalOut)
                End If
                'Send Output
                Interface3_ComWorker.WriteData(Output)
                'Sleep till next packet is needed
                Thread.Sleep(Interface3_InterfaceAssignments._PacketRateMS)
            End While
            'Shutdown
            Thread.Sleep(Interface3_InterfaceAssignments._HWStopMS)
            Interface3_ComWorker.WriteData(Interface3_InterfaceAssignments._ShutdownOutput)
            Interface3_ComWorker.ClosePort()
        Else
            'did not open
            Console.Beep(1000, 100)
            Console.Beep()
            'MsgBox("Interface1 could Not Start" & vbCrLf & "Please review your settings")
        End If
    End Sub

    Private Interface4_InterfaceAssignments As Mod_NetTools.NetClient.InterfaceAssignments
    Private Interface4_ComWorker As New Mod_Com.ComWorker
    Private Sub Interface4Startup()
        'Get Assignments
        Interface4_InterfaceAssignments = CType(GameServer.GetInterfaceAssignments4, NetClient.InterfaceAssignments)
        'See if axis is used
        If Interface4_InterfaceAssignments._ComPort = "-" Then
            'not used
            Exit Sub
        End If

        'Fix Output Strings - <ascii codes from number> - get ascii numbers
        Interface4_InterfaceAssignments._StartupOutput = ReplaceWithAsciiCode(Interface4_InterfaceAssignments._StartupOutput)
        Interface4_InterfaceAssignments._InterfaceOutput = ReplaceWithAsciiCode(Interface4_InterfaceAssignments._InterfaceOutput)
        Interface4_InterfaceAssignments._ShutdownOutput = ReplaceWithAsciiCode(Interface4_InterfaceAssignments._ShutdownOutput)

        'COM Port Assignments - set assignments to the com port
        Interface4_ComWorker.PortName = Interface4_InterfaceAssignments._ComPort
        Interface4_ComWorker.BaudRate = CStr(Interface4_InterfaceAssignments._BitsPerSec)
        Interface4_ComWorker.DataBits = CStr(Interface4_InterfaceAssignments._DataBits)
        Interface4_ComWorker.Parity = Interface4_InterfaceAssignments._ParityBits
        Interface4_ComWorker.StopBits = Interface4_InterfaceAssignments._StopBits
        'See if it opens
        If Interface4_ComWorker.OpenPort = True Then
            'We need to manipulate the final output string
            Dim Output As String
            'Startup
            Interface4_ComWorker.WriteData(Interface4_InterfaceAssignments._StartupOutput)
            Thread.Sleep(Interface4_InterfaceAssignments._HWStartMS)
            While InterfacesRunning = True
                Output = Interface4_InterfaceAssignments._InterfaceOutput
                If Output.Contains("<Axis1>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis1Output, Interface4_InterfaceAssignments._OutputBits.ToString, Interface4_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis1>", FinalOut)
                End If
                If Output.Contains("<Axis2>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis2Output, Interface4_InterfaceAssignments._OutputBits.ToString, Interface4_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis2>", FinalOut)
                End If
                If Output.Contains("<Axis3>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis3Output, Interface4_InterfaceAssignments._OutputBits.ToString, Interface4_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis3>", FinalOut)
                End If
                If Output.Contains("<Axis4>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis4Output, Interface4_InterfaceAssignments._OutputBits.ToString, Interface4_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis4>", FinalOut)
                End If
                If Output.Contains("<Axis5>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis5Output, Interface4_InterfaceAssignments._OutputBits.ToString, Interface4_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis5>", FinalOut)
                End If
                If Output.Contains("<Axis6>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis6Output, Interface4_InterfaceAssignments._OutputBits.ToString, Interface4_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis6>", FinalOut)
                End If
                'Send Output
                Interface4_ComWorker.WriteData(Output)
                'Sleep till next packet is needed
                Thread.Sleep(Interface4_InterfaceAssignments._PacketRateMS)
            End While
            'Shutdown
            Thread.Sleep(Interface4_InterfaceAssignments._HWStopMS)
            Interface4_ComWorker.WriteData(Interface4_InterfaceAssignments._ShutdownOutput)
            Interface4_ComWorker.ClosePort()
        Else
            'did not open
            Console.Beep(1000, 100)
            Console.Beep()
            'MsgBox("Interface1 could Not Start" & vbCrLf & "Please review your settings")
        End If
    End Sub

    Private Interface5_InterfaceAssignments As Mod_NetTools.NetClient.InterfaceAssignments
    Private Interface5_ComWorker As New Mod_Com.ComWorker
    Private Sub Interface5Startup()
        'Get Assignments
        Interface5_InterfaceAssignments = CType(GameServer.GetInterfaceAssignments5, NetClient.InterfaceAssignments)
        'See if axis is used
        If Interface5_InterfaceAssignments._ComPort = "-" Then
            'not used
            Exit Sub
        End If

        'Fix Output Strings - <ascii codes from number> - get ascii numbers
        Interface5_InterfaceAssignments._StartupOutput = ReplaceWithAsciiCode(Interface5_InterfaceAssignments._StartupOutput)
        Interface5_InterfaceAssignments._InterfaceOutput = ReplaceWithAsciiCode(Interface5_InterfaceAssignments._InterfaceOutput)
        Interface5_InterfaceAssignments._ShutdownOutput = ReplaceWithAsciiCode(Interface5_InterfaceAssignments._ShutdownOutput)

        'COM Port Assignments - set assignments to the com port
        Interface5_ComWorker.PortName = Interface5_InterfaceAssignments._ComPort
        Interface5_ComWorker.BaudRate = CStr(Interface5_InterfaceAssignments._BitsPerSec)
        Interface5_ComWorker.DataBits = CStr(Interface5_InterfaceAssignments._DataBits)
        Interface5_ComWorker.Parity = Interface5_InterfaceAssignments._ParityBits
        Interface5_ComWorker.StopBits = Interface5_InterfaceAssignments._StopBits
        'See if it opens
        If Interface5_ComWorker.OpenPort = True Then
            'We need to manipulate the final output string
            Dim Output As String
            'Startup
            Interface5_ComWorker.WriteData(Interface5_InterfaceAssignments._StartupOutput)
            Thread.Sleep(Interface5_InterfaceAssignments._HWStartMS)
            While InterfacesRunning = True
                Output = Interface5_InterfaceAssignments._InterfaceOutput
                If Output.Contains("<Axis1>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis1Output, Interface5_InterfaceAssignments._OutputBits.ToString, Interface5_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis1>", FinalOut)
                End If
                If Output.Contains("<Axis2>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis2Output, Interface5_InterfaceAssignments._OutputBits.ToString, Interface5_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis2>", FinalOut)
                End If
                If Output.Contains("<Axis3>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis3Output, Interface5_InterfaceAssignments._OutputBits.ToString, Interface5_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis3>", FinalOut)
                End If
                If Output.Contains("<Axis4>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis4Output, Interface5_InterfaceAssignments._OutputBits.ToString, Interface5_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis4>", FinalOut)
                End If
                If Output.Contains("<Axis5>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis5Output, Interface5_InterfaceAssignments._OutputBits.ToString, Interface5_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis5>", FinalOut)
                End If
                If Output.Contains("<Axis6>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis6Output, Interface5_InterfaceAssignments._OutputBits.ToString, Interface5_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis6>", FinalOut)
                End If
                'Send Output
                Interface5_ComWorker.WriteData(Output)
                'Sleep till next packet is needed
                Thread.Sleep(Interface5_InterfaceAssignments._PacketRateMS)
            End While
            'Shutdown
            Thread.Sleep(Interface5_InterfaceAssignments._HWStopMS)
            Interface5_ComWorker.WriteData(Interface5_InterfaceAssignments._ShutdownOutput)
            Interface5_ComWorker.ClosePort()
        Else
            'did not open
            Console.Beep(1000, 100)
            Console.Beep()
            'MsgBox("Interface1 could Not Start" & vbCrLf & "Please review your settings")
        End If
    End Sub

    Private Interface6_InterfaceAssignments As Mod_NetTools.NetClient.InterfaceAssignments
    Private Interface6_ComWorker As New Mod_Com.ComWorker
    Private Sub Interface6Startup()
        'Get Assignments
        Interface6_InterfaceAssignments = CType(GameServer.GetInterfaceAssignments6, NetClient.InterfaceAssignments)
        'See if axis is used
        If Interface6_InterfaceAssignments._ComPort = "-" Then
            'not used
            Exit Sub
        End If

        'Fix Output Strings - <ascii codes from number> - get ascii numbers
        Interface6_InterfaceAssignments._StartupOutput = ReplaceWithAsciiCode(Interface6_InterfaceAssignments._StartupOutput)
        Interface6_InterfaceAssignments._InterfaceOutput = ReplaceWithAsciiCode(Interface6_InterfaceAssignments._InterfaceOutput)
        Interface6_InterfaceAssignments._ShutdownOutput = ReplaceWithAsciiCode(Interface6_InterfaceAssignments._ShutdownOutput)

        'COM Port Assignments - set assignments to the com port
        Interface6_ComWorker.PortName = Interface6_InterfaceAssignments._ComPort
        Interface6_ComWorker.BaudRate = CStr(Interface6_InterfaceAssignments._BitsPerSec)
        Interface6_ComWorker.DataBits = CStr(Interface6_InterfaceAssignments._DataBits)
        Interface6_ComWorker.Parity = Interface6_InterfaceAssignments._ParityBits
        Interface6_ComWorker.StopBits = Interface6_InterfaceAssignments._StopBits
        'See if it opens
        If Interface6_ComWorker.OpenPort = True Then
            'We need to manipulate the final output string
            Dim Output As String
            'Startup
            Interface6_ComWorker.WriteData(Interface6_InterfaceAssignments._StartupOutput)
            Thread.Sleep(Interface6_InterfaceAssignments._HWStartMS)
            While InterfacesRunning = True
                Output = Interface6_InterfaceAssignments._InterfaceOutput
                If Output.Contains("<Axis1>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis1Output, Interface6_InterfaceAssignments._OutputBits.ToString, Interface6_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis1>", FinalOut)
                End If
                If Output.Contains("<Axis2>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis2Output, Interface6_InterfaceAssignments._OutputBits.ToString, Interface6_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis2>", FinalOut)
                End If
                If Output.Contains("<Axis3>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis3Output, Interface6_InterfaceAssignments._OutputBits.ToString, Interface6_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis3>", FinalOut)
                End If
                If Output.Contains("<Axis4>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis4Output, Interface6_InterfaceAssignments._OutputBits.ToString, Interface6_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis4>", FinalOut)
                End If
                If Output.Contains("<Axis5>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis5Output, Interface6_InterfaceAssignments._OutputBits.ToString, Interface6_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis5>", FinalOut)
                End If
                If Output.Contains("<Axis6>") = True Then
                    Dim FinalOut As String = GetOutPut(GameServer.Get_Axis6Output, Interface6_InterfaceAssignments._OutputBits.ToString, Interface6_InterfaceAssignments._OutputType)
                    Output = Output.Replace("<Axis6>", FinalOut)
                End If
                'Send Output
                Interface6_ComWorker.WriteData(Output)
                'Sleep till next packet is needed
                Thread.Sleep(Interface6_InterfaceAssignments._PacketRateMS)
            End While
            'Shutdown
            Thread.Sleep(Interface6_InterfaceAssignments._HWStopMS)
            Interface6_ComWorker.WriteData(Interface6_InterfaceAssignments._ShutdownOutput)
            Interface6_ComWorker.ClosePort()
        Else
            'did not open
            Console.Beep(1000, 100)
            Console.Beep()
            'MsgBox("Interface1 could Not Start" & vbCrLf & "Please review your settings")
        End If
    End Sub
#End Region

    Private Function ReplaceWithAsciiCode(InputString As String) As String
        If InputString = "-" Or InputString = "" Then
            Return InputString
        End If
        For x = 0 To 255
            Dim SearchString As String = ("<" & x.ToString & ">").ToString
            If InputString.Contains(SearchString) Then
                Dim ReplaceString As String = (Chr(x)).ToString
                InputString = InputString.Replace(SearchString, ReplaceString)
            End If
        Next
        Return InputString
    End Function
End Module
