﻿Option Explicit On
Option Strict On

Imports System.Net.Sockets
Imports System.Xml.Serialization
Imports System.IO
Imports System.Text
Imports System.Threading
Imports System.Net

Public Module Mod_NetTools
    Public Class NetClient
        Public Event MessageWaiting()
        Private MyUDPClient As New UdpClient
        Private MyTCPClient As New TcpClient
        Private Message As String
        Private MyPort As Integer
        Private MyIp As String

        Sub SetPort(InPort As Integer)
            MyPort = InPort
        End Sub

        Sub SetIp(InIPAddress As String)
            MyIp = InIPAddress
        End Sub

        Sub UDPOpen()
            Try
                MyUDPClient.Connect(MyIp, MyPort)
            Catch ex As Exception
                'MsgBox(ex.Message.ToString)
            End Try
        End Sub

        Function TcpOpen() As Boolean
            Try
                MyTCPClient = New TcpClient(MyIp, MyPort)
                MyTCPClient.ReceiveTimeout = 1
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function

        Sub UDPClose()
            Try
                MyUDPClient.Close()
                receivingUdpClient.Close()
                ThreadReceive.Abort()
                'receivingUdpClient.Close()
            Catch ex As Exception
                'MsgBox(ex.Message.ToString)
            End Try
        End Sub

        Sub TcpClose()
            Try
                MyTCPClient.Close()
                ListenerThread.Abort()
            Catch ex As Exception
                'MsgBox(ex.Message.ToString)
            End Try
        End Sub

        Sub SendMessageUDP(WhatToSend As String)
            Try
                Dim bytCommand As Byte() = New Byte() {}
                bytCommand = Encoding.ASCII.GetBytes(WhatToSend)
                MyUDPClient.Send(bytCommand, bytCommand.Length)
            Catch ex As Exception
                'MsgBox(ex.Message.ToString)
            End Try
        End Sub

        Sub SendMessageTCP(WhatToSend As String)
            Try
                Dim Writer As New StreamWriter(MyTCPClient.GetStream())
                Writer.Write(WhatToSend)
                Writer.Flush()
            Catch ex As Exception
                'MsgBox(ex.Message.ToString)
            End Try
        End Sub

        'UDP SERVER STUFF
        Private receivingUdpClient As New UdpClient
        Private RemoteIpEndPoint As New IPEndPoint(System.Net.IPAddress.Any, 0)
        Private ThreadReceive As Thread
        Sub StartUDPListener()
            receivingUdpClient = New UdpClient(MyPort)
            ThreadReceive = New Thread(AddressOf ReceiveMessages)
            ThreadReceive.Start()
        End Sub

        Function GetMessage() As String
            Return Message
        End Function

        Private Sub ReceiveMessages()
            Dim receiveBytes As [Byte]() = receivingUdpClient.Receive(RemoteIpEndPoint)

            'get message
            Message = System.Text.Encoding.ASCII.GetString(receiveBytes)
            RaiseEvent MessageWaiting()


            'Start New Thread
            ThreadReceive = New System.Threading.Thread(AddressOf ReceiveMessages)
            ThreadReceive.Start()
        End Sub

        'TCP SERVER STUFF
        Private Listener As New TcpListener(0)
        Private Sub Listening()
            If MyIp = "Any" Then
                Listener = New TcpListener(System.Net.IPAddress.Any, MyPort)
            Else
                Listener = New TcpListener(System.Net.IPAddress.Parse(MyIp), MyPort)
            End If

            Listener.Start()

retry:
            If Listener.Pending = True Then
                Dim IncommingMessage As String = ""
                MyTCPClient = Listener.AcceptTcpClient()

                Dim Reader As New StreamReader(MyTCPClient.GetStream())
                While Reader.Peek > -1
                    IncommingMessage = IncommingMessage + Convert.ToChar(Reader.Read()).ToString
                End While

                Message = IncommingMessage
                RaiseEvent MessageWaiting()
            End If
            Thread.Sleep(100)
            GoTo retry
        End Sub

        Dim ListenerThread As Thread
        Sub StartTCPListener()
            Dim ListenerThread As New Thread(New ThreadStart(AddressOf Listening))
            ListenerThread.Start()
        End Sub

        Public Structure ProfileData
            Public _GameName As String
            Public _ProfileName As String
            Public _MainLevel As Integer
            Public _RollEditor As Integer
            Public _PitchEditor As Integer
            Public _HeaveEditor As Integer
            Public _YawEditor As Integer
            Public _SwayEditor As Integer
            Public _SurgeEditor As Integer
            Public _RollSmoothingFilter As Integer
            Public _PitchSmoothingFilter As Integer
            Public _HeaveSmoothingFilter As Integer
            Public _YawSmoothingFilter As Integer
            Public _SwaySmoothingFilter As Integer
            Public _SurgeSmoothingFilter As Integer
            Public _RollClippingFilter As Integer
            Public _PitchClippingFilter As Integer
            Public _HeaveClippingFilter As Integer
            Public _YawClippingFilter As Integer
            Public _SwayClippingFilter As Integer
            Public _SurgeClippingFilter As Integer
            Public _RollWashoutFilter As Integer
            Public _PitchWashoutFilter As Integer
            Public _HeaveWashoutFilter As Integer
            Public _YawWashoutFilter As Integer
            Public _SwayWashoutFilter As Integer
            Public _SurgeWashoutFilter As Integer
        End Structure

        Public Structure AxisAssignments
            Public _Axis1DIR1 As Boolean
            Public _Axis1DOF1 As String
            Public _Axis1PER1 As Integer
            Public _Axis1DIR2 As Boolean
            Public _Axis1DOF2 As String
            Public _Axis1PER2 As Integer
            Public _Axis1DIR3 As Boolean
            Public _Axis1DOF3 As String
            Public _Axis1PER3 As Integer

            Public _Axis2DIR1 As Boolean
            Public _Axis2DOF1 As String
            Public _Axis2PER1 As Integer
            Public _Axis2DIR2 As Boolean
            Public _Axis2DOF2 As String
            Public _Axis2PER2 As Integer
            Public _Axis2DIR3 As Boolean
            Public _Axis2DOF3 As String
            Public _Axis2PER3 As Integer

            Public _Axis3DIR1 As Boolean
            Public _Axis3DOF1 As String
            Public _Axis3PER1 As Integer
            Public _Axis3DIR2 As Boolean
            Public _Axis3DOF2 As String
            Public _Axis3PER2 As Integer
            Public _Axis3DIR3 As Boolean
            Public _Axis3DOF3 As String
            Public _Axis3PER3 As Integer

            Public _Axis4DIR1 As Boolean
            Public _Axis4DOF1 As String
            Public _Axis4PER1 As Integer
            Public _Axis4DIR2 As Boolean
            Public _Axis4DOF2 As String
            Public _Axis4PER2 As Integer
            Public _Axis4DIR3 As Boolean
            Public _Axis4DOF3 As String
            Public _Axis4PER3 As Integer

            Public _Axis5DIR1 As Boolean
            Public _Axis5DOF1 As String
            Public _Axis5PER1 As Integer
            Public _Axis5DIR2 As Boolean
            Public _Axis5DOF2 As String
            Public _Axis5PER2 As Integer
            Public _Axis5DIR3 As Boolean
            Public _Axis5DOF3 As String
            Public _Axis5PER3 As Integer

            Public _Axis6DIR1 As Boolean
            Public _Axis6DOF1 As String
            Public _Axis6PER1 As Integer
            Public _Axis6DIR2 As Boolean
            Public _Axis6DOF2 As String
            Public _Axis6PER2 As Integer
            Public _Axis6DIR3 As Boolean
            Public _Axis6DOF3 As String
            Public _Axis6PER3 As Integer
        End Structure

        Public Structure AxisLimiting
            Public _Axis1Limiter As Integer
            Public _Axis2Limiter As Integer
            Public _Axis3Limiter As Integer
            Public _Axis4Limiter As Integer
            Public _Axis5Limiter As Integer
            Public _Axis6Limiter As Integer
        End Structure

        Public Structure InterfaceAssignments
            Public _InterfaceNumber As Integer
            Public _ComPort As String
            Public _BitsPerSec As Integer
            Public _DataBits As Integer
            Public _ParityBits As String
            Public _StopBits As String
            Public _OutputBits As Integer
            Public _OutputType As String
            Public _StartupOutput As String
            Public _HWStartMS As Integer
            Public _InterfaceOutput As String
            Public _PacketRateMS As Integer
            Public _ShutdownOutput As String
            Public _HWStopMS As Integer
        End Structure

        Public Structure OutputTesting
            Public _OutputType As String
            Public _Axis1 As Double
            Public _Axis2 As Double
            Public _Axis3 As Double
            Public _Axis4 As Double
            Public _Axis5 As Double
            Public _Axis6 As Double
            Public _Roll As Double
            Public _Pitch As Double
            Public _Heave As Double
            Public _Yaw As Double
            Public _Sway As Double
            Public _Surge As Double
        End Structure

        Public Structure VirtualAxis
            Public _Axis1 As Integer
            Public _Axis2 As Integer
            Public _Axis3 As Integer
            Public _Axis4 As Integer
            Public _Axis5 As Integer
            Public _Axis6 As Integer
            Public _Roll As Integer
            Public _Pitch As Integer
            Public _Heave As Integer
            Public _Yaw As Integer
            Public _Sway As Integer
            Public _Surge As Integer
        End Structure

        Public Structure GameMaxMin
            Public _GameName As String
            Public _RollMax As Double
            Public _RollMin As Double
            Public _PitchMax As Double
            Public _PitchMin As Double
            Public _HeaveMax As Double
            Public _HeaveMin As Double
            Public _YawMax As Double
            Public _YawMin As Double
            Public _SwayMax As Double
            Public _SwayMin As Double
            Public _SurgeMax As Double
            Public _SurgeMin As Double
        End Structure

        Public Structure GameAdminData
            Public _InputRoll As Double
            Public _InputPitch As Double
            Public _InputHeave As Double
            Public _InputYaw As Double
            Public _InputSway As Double
            Public _InputSurge As Double
            Public _OutputRoll As Double
            Public _OutputPitch As Double
            Public _OutputHeave As Double
            Public _OutputYaw As Double
            Public _OutputSway As Double
            Public _OutputSurge As Double
            Public _Axis1Output As Double
            Public _Axis2Output As Double
            Public _Axis3Output As Double
            Public _Axis4Output As Double
            Public _Axis5Output As Double
            Public _Axis6Output As Double
        End Structure

        Public Structure GenericOutput
            Public Roll As Single
            Public Pitch As Single
            Public Heave As Single
            Public Yaw As Single
            Public Sway As Single
            Public Surge As Single
        End Structure

        Public Function SerializeString(StructureType As String, MyStructure As Object) As String
            Dim string_writer As New StringWriter
            Dim SerializedOutput As String
            Select Case StructureType
                Case "GenericOutput"
                    Dim xml_serializer As New XmlSerializer(GetType(GenericOutput))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "ProfileData"
                    Dim xml_serializer As New XmlSerializer(GetType(ProfileData))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "AxisAssignments"
                    Dim xml_serializer As New XmlSerializer(GetType(AxisAssignments))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "AxisLimiting"
                    Dim xml_serializer As New XmlSerializer(GetType(AxisLimiting))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "InterfaceAssignments"
                    Dim xml_serializer As New XmlSerializer(GetType(InterfaceAssignments))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "OutputTesting"
                    Dim xml_serializer As New XmlSerializer(GetType(OutputTesting))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "VirtualAxis"
                    Dim xml_serializer As New XmlSerializer(GetType(VirtualAxis))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "GameMaxMin"
                    Dim xml_serializer As New XmlSerializer(GetType(GameMaxMin))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case "GameAdminData"
                    Dim xml_serializer As New XmlSerializer(GetType(GameAdminData))
                    xml_serializer.Serialize(string_writer, MyStructure)
                Case Else
                    SerializedOutput = ""
                    string_writer.Close()
                    Return SerializedOutput
                    Exit Function
            End Select
            SerializedOutput = string_writer.ToString()
            string_writer.Close()
            Return SerializedOutput
        End Function

        Public Function DeSerializeString(InputString As String, StructureType As String) As Object
            Select Case StructureType
                Case "GenericOutput"
                    On Error Resume Next
                    Dim xml_serializer As New XmlSerializer(GetType(GenericOutput))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As GenericOutput = DirectCast(xml_serializer.Deserialize(string_reader), GenericOutput)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "ProfileData"
                    Dim xml_serializer As New XmlSerializer(GetType(ProfileData))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As ProfileData = DirectCast(xml_serializer.Deserialize(string_reader), ProfileData)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "AxisAssignments"
                    Dim xml_serializer As New XmlSerializer(GetType(AxisAssignments))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As AxisAssignments = DirectCast(xml_serializer.Deserialize(string_reader), AxisAssignments)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "AxisLimiting"
                    Dim xml_serializer As New XmlSerializer(GetType(AxisLimiting))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As AxisLimiting = DirectCast(xml_serializer.Deserialize(string_reader), AxisLimiting)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "InterfaceAssignments"
                    Dim xml_serializer As New XmlSerializer(GetType(InterfaceAssignments))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As InterfaceAssignments = DirectCast(xml_serializer.Deserialize(string_reader), InterfaceAssignments)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "OutputTesting"
                    Dim xml_serializer As New XmlSerializer(GetType(OutputTesting))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As OutputTesting = DirectCast(xml_serializer.Deserialize(string_reader), OutputTesting)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "VirtualAxis"
                    Dim xml_serializer As New XmlSerializer(GetType(VirtualAxis))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As VirtualAxis = DirectCast(xml_serializer.Deserialize(string_reader), VirtualAxis)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "GameMaxMin"
                    Dim xml_serializer As New XmlSerializer(GetType(GameMaxMin))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As GameMaxMin = DirectCast(xml_serializer.Deserialize(string_reader), GameMaxMin)
                    string_reader.Close()
                    Return DeserializedOutput
                Case "GameAdminData"
                    Dim xml_serializer As New XmlSerializer(GetType(GameAdminData))
                    Dim string_reader As New StringReader(InputString)
                    Dim DeserializedOutput As GameAdminData = DirectCast(xml_serializer.Deserialize(string_reader), GameAdminData)
                    string_reader.Close()
                    Return DeserializedOutput
                Case Else
                    Dim DeserializedOutput As Object = vbNull
                    Return DeserializedOutput
            End Select
        End Function
    End Class
End Module
